@@ -22,7 +22,7 @@ use crate::common::progress_stop_handler::{check_if_stop_received, prepare_threa
 use crate::common::tool_data::CommonToolData;
 use crate::flc;
 
-#[derive(Copy, Clone, Eq, PartialEq)]
+#[derive(Copy, Clone, Eq, PartialEq, Debug)]
 pub enum Collect {
     InvalidSymlinks,
     Files,
@@ -39,6 +39,7 @@ enum EntryType {
 pub struct DirTraversalBuilder<'b, F> {
     group_by: Option<F>,
     root_dirs: Vec<PathBuf>,
+    root_files: Vec<PathBuf>,
     stop_flag: Option<Arc<AtomicBool>>,
     progress_sender: Option<&'b Sender<ProgressData>>,
     minimal_file_size: Option<u64>,
@@ -52,9 +53,11 @@ pub struct DirTraversalBuilder<'b, F> {
     tool_type: ToolType,
 }
 
+#[derive(Debug)]
 pub struct DirTraversal<'b, F> {
     group_by: F,
     root_dirs: Vec<PathBuf>,
+    root_files: Vec<PathBuf>,
     stop_flag: Arc<AtomicBool>,
     progress_sender: Option<&'b Sender<ProgressData>>,
     recursive_search: bool,
@@ -78,7 +81,8 @@ impl DirTraversalBuilder<'_, ()> {
     pub fn new() -> Self {
         DirTraversalBuilder {
             group_by: None,
-            root_dirs: vec![],
+            root_dirs: Vec::new(),
+            root_files: Vec::new(),
             stop_flag: None,
             progress_sender: None,
             checking_method: CheckingMethod::None,
@@ -97,6 +101,7 @@ impl DirTraversalBuilder<'_, ()> {
 impl<'b, F> DirTraversalBuilder<'b, F> {
     pub(crate) fn common_data(mut self, common_tool_data: &CommonToolData) -> Self {
         self.root_dirs = common_tool_data.directories.included_directories.clone();
+        self.root_files = common_tool_data.directories.included_files.clone();
         self.extensions = Some(common_tool_data.extensions.clone());
         self.excluded_items = Some(common_tool_data.excluded_items.clone());
         self.recursive_search = common_tool_data.recursive_search;
@@ -144,6 +149,7 @@ impl<'b, F> DirTraversalBuilder<'b, F> {
         DirTraversalBuilder {
             group_by: Some(group_by),
             root_dirs: self.root_dirs,
+            root_files: self.root_files,
             stop_flag: self.stop_flag,
             progress_sender: self.progress_sender,
             directories: self.directories,
@@ -162,6 +168,7 @@ impl<'b, F> DirTraversalBuilder<'b, F> {
         DirTraversal {
             group_by: self.group_by.expect("could not build"),
             root_dirs: self.root_dirs,
+            root_files: self.root_files,
             stop_flag: self.stop_flag.expect("Stop flag must be always initialized"),
             progress_sender: self.progress_sender,
             checking_method: self.checking_method,
@@ -206,11 +213,12 @@ where
     pub(crate) fn run(self) -> DirTraversalResult<T> {
         assert_ne!(self.tool_type, ToolType::None, "Tool type cannot be None");
 
-        let mut all_warnings = vec![];
+        let mut all_warnings = Vec::new();
         let mut grouped_file_entries: BTreeMap<T, Vec<FileEntry>> = BTreeMap::new();
 
-        // Add root folders for finding
+        // Add root folders and files for finding
         let mut folders_to_check: Vec<PathBuf> = self.root_dirs.clone();
+        let mut files_to_check: Vec<PathBuf> = self.root_files.clone();
 
         let progress_handler = prepare_thread_handler_common(self.progress_sender, CurrentStage::CollectingFiles, 0, (self.tool_type, self.checking_method), 0);
 
@@ -225,6 +233,47 @@ where
             stop_flag,
             ..
         } = self;
+
+        let mut file_results = Vec::new();
+        // File traversal
+        while let Some(current_file) = files_to_check.pop() {
+            let Some(metadata) = common_get_metadata_from_path(&current_file, &mut all_warnings) else {
+                continue;
+            };
+            let file_type = metadata.file_type();
+            match (entry_type(file_type), collect) {
+                (EntryType::File, Collect::Files) => {
+                    progress_handler.increase_items(1);
+                    process_file_in_file_mode_path_check(
+                        &current_file,
+                        &metadata,
+                        &mut all_warnings,
+                        &mut file_results,
+                        &extensions,
+                        &excluded_items,
+                        minimal_file_size,
+                        maximal_file_size,
+                    );
+                }
+                (EntryType::File, Collect::InvalidSymlinks) => {
+                    progress_handler.increase_items(1);
+                }
+                (EntryType::Symlink, Collect::InvalidSymlinks) => {
+                    progress_handler.increase_items(1);
+                    process_symlink_in_symlink_mode_path_check(&current_file, &metadata, &mut all_warnings, &mut file_results, &extensions, &excluded_items);
+                }
+                (EntryType::Symlink | EntryType::Dir | EntryType::Other, _) => {
+                    // nothing to do
+                }
+            }
+        }
+        file_results.sort_by_cached_key(|fe| fe.path.to_string_lossy().to_string());
+        for fe in file_results {
+            let key = (self.group_by)(&fe);
+            grouped_file_entries.entry(key).or_default().push(fe);
+        }
+
+        // Folder traversal
         while !folders_to_check.is_empty() {
             if check_if_stop_received(&stop_flag) {
                 progress_handler.join_thread();
@@ -235,9 +284,9 @@ where
                 .into_par_iter()
                 .with_max_len(2) // Avoiding checking too many folders in batch
                 .map(|current_folder| {
-                    let mut dir_result = vec![];
-                    let mut warnings = vec![];
-                    let mut fe_result = vec![];
+                    let mut dir_result = Vec::new();
+                    let mut warnings = Vec::new();
+                    let mut fe_result = Vec::new();
 
                     let Some(read_dir) = common_read_dir(&current_folder, &mut warnings) else {
                         return Some((dir_result, warnings, fe_result));
@@ -331,7 +380,7 @@ fn process_file_in_file_mode(
     minimal_file_size: u64,
     maximal_file_size: u64,
 ) {
-    if !extensions.check_if_entry_have_valid_extension(entry_data) {
+    if !extensions.check_if_entry_have_valid_extension(&entry_data.file_name()) {
         return;
     }
 
@@ -349,6 +398,9 @@ fn process_file_in_file_mode(
         }
     }
 
+    #[cfg(windows)]
+    let _ = directories; // Silence unused variable warning on Windows
+
     let Some(metadata) = common_get_metadata_dir(entry_data, warnings, &current_file_name) else {
         return;
     };
@@ -364,6 +416,40 @@ fn process_file_in_file_mode(
         fe_result.push(fe);
     }
 }
+// Same as above, but working with Path instead of DirEntry
+// Sadly this cannot be merged, due to a little crazy optimizations done in this functions
+fn process_file_in_file_mode_path_check(
+    path: &Path,
+    metadata: &Metadata,
+    warnings: &mut Vec<String>,
+    fe_result: &mut Vec<FileEntry>,
+    extensions: &Extensions,
+    excluded_items: &ExcludedItems,
+    minimal_file_size: u64,
+    maximal_file_size: u64,
+) {
+    let Some(file_name) = path.file_name() else {
+        return;
+    };
+    if !extensions.check_if_entry_have_valid_extension(file_name) {
+        return;
+    }
+
+    if excluded_items.is_excluded(path) {
+        return;
+    }
+
+    if (minimal_file_size..=maximal_file_size).contains(&metadata.len()) {
+        // Creating new file entry
+        let fe: FileEntry = FileEntry {
+            size: metadata.len(),
+            modified_date: get_modified_time(metadata, warnings, path, false),
+            path: path.to_path_buf(),
+        };
+
+        fe_result.push(fe);
+    }
+}
 
 fn process_dir_in_file_symlink_mode(
     recursive_search: bool,
@@ -395,6 +481,9 @@ fn process_dir_in_file_symlink_mode(
         }
     }
 
+    #[cfg(windows)]
+    let _ = warnings; // Silence unused variable warning on Windows
+
     dir_result.push(dir_path);
 }
 
@@ -406,7 +495,7 @@ fn process_symlink_in_symlink_mode(
     directories: &Directories,
     excluded_items: &ExcludedItems,
 ) {
-    if !extensions.check_if_entry_have_valid_extension(entry_data) {
+    if !extensions.check_if_entry_have_valid_extension(&entry_data.file_name()) {
         return;
     }
 
@@ -424,6 +513,9 @@ fn process_symlink_in_symlink_mode(
         }
     }
 
+    #[cfg(windows)]
+    let _ = directories; // Silence unused variable warning on Windows
+
     let Some(metadata) = common_get_metadata_dir(entry_data, warnings, &current_file_name) else {
         return;
     };
@@ -437,6 +529,34 @@ fn process_symlink_in_symlink_mode(
 
     fe_result.push(fe);
 }
+fn process_symlink_in_symlink_mode_path_check(
+    path: &Path,
+    metadata: &Metadata,
+    warnings: &mut Vec<String>,
+    fe_result: &mut Vec<FileEntry>,
+    extensions: &Extensions,
+    excluded_items: &ExcludedItems,
+) {
+    let Some(file_name) = path.file_name() else {
+        return;
+    };
+    if !extensions.check_if_entry_have_valid_extension(file_name) {
+        return;
+    }
+
+    if excluded_items.is_excluded(path) {
+        return;
+    }
+
+    // Creating new file entry
+    let fe: FileEntry = FileEntry {
+        size: metadata.len(),
+        modified_date: get_modified_time(metadata, warnings, path, false),
+        path: path.to_path_buf(),
+    };
+
+    fe_result.push(fe);
+}
 
 pub(crate) fn common_read_dir(current_folder: &Path, warnings: &mut Vec<String>) -> Option<Vec<Result<DirEntry, std::io::Error>>> {
     match fs::read_dir(current_folder) {
@@ -476,6 +596,17 @@ pub(crate) fn common_get_metadata_dir(entry_data: &DirEntry, warnings: &mut Vec<
     Some(metadata)
 }
 
+pub(crate) fn common_get_metadata_from_path(path: &Path, warnings: &mut Vec<String>) -> Option<Metadata> {
+    let metadata: Metadata = match fs::metadata(path) {
+        Ok(t) => t,
+        Err(e) => {
+            warnings.push(flc!("core_cannot_read_metadata_file", file = path.to_string_lossy().to_string(), reason = e.to_string()));
+            return None;
+        }
+    };
+    Some(metadata)
+}
+
 pub(crate) fn get_modified_time(metadata: &Metadata, warnings: &mut Vec<String>, current_file_name: &Path, is_folder: bool) -> u64 {
     match metadata.modified() {
         Ok(t) => match t.duration_since(UNIX_EPOCH) {
@@ -527,13 +658,12 @@ pub(crate) fn take_1_per_inode((k, mut v): (Option<u64>, Vec<FileEntry>)) -> Vec
 
 #[cfg(test)]
 mod tests {
-    use std::collections::HashSet;
     use std::fs::File;
     use std::io::prelude::*;
     use std::time::{Duration, SystemTime};
     use std::{fs, io};
 
-    use once_cell::sync::Lazy;
+    use indexmap::IndexSet;
     use tempfile::TempDir;
 
     use super::*;
@@ -550,12 +680,12 @@ mod tests {
         fn get_cd_mut(&mut self) -> &mut CommonToolData {
             self
         }
-        fn found_any_broken_files(&self) -> bool {
+        fn found_any_items(&self) -> bool {
             false
         }
     }
 
-    static NOW: Lazy<SystemTime> = Lazy::new(|| SystemTime::UNIX_EPOCH + Duration::new(100, 0));
+    static NOW: std::sync::LazyLock<SystemTime> = std::sync::LazyLock::new(|| SystemTime::UNIX_EPOCH + Duration::new(100, 0));
     const CONTENT: &[u8; 1] = b"a";
 
     fn create_files(dir: &TempDir) -> io::Result<(PathBuf, PathBuf, PathBuf)> {
@@ -579,7 +709,7 @@ mod tests {
         let secs = NOW.duration_since(SystemTime::UNIX_EPOCH).expect("Cannot fail calculating duration since epoch").as_secs();
 
         let mut common_data = CommonToolData::new(ToolType::SimilarImages);
-        common_data.directories.set_included_directory([dir.path().to_owned()].to_vec());
+        common_data.directories.set_included_paths([dir.path().to_owned()].to_vec());
         common_data.set_minimal_file_size(0);
 
         match DirTraversalBuilder::new()
@@ -593,9 +723,9 @@ mod tests {
                 warnings: _,
                 grouped_file_entries,
             } => {
-                let actual: HashSet<_> = grouped_file_entries.into_values().flatten().collect();
+                let actual: IndexSet<_> = grouped_file_entries.into_values().flatten().collect();
                 assert_eq!(
-                    HashSet::from([
+                    IndexSet::from([
                         FileEntry {
                             path: src,
                             size: 1,
@@ -618,7 +748,7 @@ mod tests {
             DirTraversalResult::Stopped => {
                 panic!("Expect SuccessFiles.");
             }
-        };
+        }
         Ok(())
     }
 
@@ -630,7 +760,7 @@ mod tests {
         let secs = NOW.duration_since(SystemTime::UNIX_EPOCH).expect("Cannot fail calculating duration since epoch").as_secs();
 
         let mut common_data = CommonToolData::new(ToolType::SimilarImages);
-        common_data.directories.set_included_directory([dir.path().to_owned()].to_vec());
+        common_data.directories.set_included_paths([dir.path().to_owned()].to_vec());
         common_data.set_minimal_file_size(0);
 
         match DirTraversalBuilder::new()
@@ -644,9 +774,9 @@ mod tests {
                 warnings: _,
                 grouped_file_entries,
             } => {
-                let actual: HashSet<_> = grouped_file_entries.into_iter().flat_map(take_1_per_inode).collect();
+                let actual: IndexSet<_> = grouped_file_entries.into_iter().flat_map(take_1_per_inode).collect();
                 assert_eq!(
-                    HashSet::from([
+                    IndexSet::from([
                         FileEntry {
                             path: src,
                             size: 1,
@@ -664,7 +794,7 @@ mod tests {
             DirTraversalResult::Stopped => {
                 panic!("Expect SuccessFiles.");
             }
-        };
+        }
         Ok(())
     }
 
@@ -676,7 +806,7 @@ mod tests {
         let secs = NOW.duration_since(SystemTime::UNIX_EPOCH).expect("Cannot fail duration from epoch").as_secs();
 
         let mut common_data = CommonToolData::new(ToolType::SimilarImages);
-        common_data.directories.set_included_directory([dir.path().to_owned()].to_vec());
+        common_data.directories.set_included_paths([dir.path().to_owned()].to_vec());
         common_data.set_minimal_file_size(0);
 
         match DirTraversalBuilder::new()
@@ -690,9 +820,9 @@ mod tests {
                 warnings: _,
                 grouped_file_entries,
             } => {
-                let actual: HashSet<_> = grouped_file_entries.into_iter().flat_map(take_1_per_inode).collect();
+                let actual: IndexSet<_> = grouped_file_entries.into_iter().flat_map(take_1_per_inode).collect();
                 assert_eq!(
-                    HashSet::from([
+                    IndexSet::from([
                         FileEntry {
                             path: src,
                             size: 1,
