@@ -2,14 +2,13 @@ use std::collections::HashMap;
 
 use i18n_embed::fluent::{FluentLanguageLoader, fluent_language_loader};
 use i18n_embed::{DefaultLocalizer, LanguageLoader, Localizer};
-use once_cell::sync::Lazy;
 use rust_embed::RustEmbed;
 
 #[derive(RustEmbed)]
 #[folder = "i18n/"]
 struct Localizations;
 
-pub static LANGUAGE_LOADER_CORE: Lazy<FluentLanguageLoader> = Lazy::new(|| {
+pub static LANGUAGE_LOADER_CORE: std::sync::LazyLock<FluentLanguageLoader> = std::sync::LazyLock::new(|| {
     let loader: FluentLanguageLoader = fluent_language_loader!();
 
     loader.load_fallback_language(&Localizations).expect("Error while loading fallback language");
@@ -19,17 +18,11 @@ pub static LANGUAGE_LOADER_CORE: Lazy<FluentLanguageLoader> = Lazy::new(|| {
 
 #[macro_export]
 macro_rules! flc {
-    ($message_id:literal) => {{
-        i18n_embed_fl::fl!($crate::localizer_core::LANGUAGE_LOADER_CORE, $message_id)
-    }};
-
-    ($message_id:literal, $($args:expr),*) => {{
-        i18n_embed_fl::fl!($crate::localizer_core::LANGUAGE_LOADER_CORE, $message_id, $($args), *)
+    ( $($tt:tt)* ) => {{
+        i18n_embed_fl::fl!($crate::localizer_core::LANGUAGE_LOADER_CORE, $($tt)*)
     }};
 }
 
-// Get the `Localizer` to be used for localizing this library.
-
 pub fn localizer_core() -> Box<dyn Localizer> {
     Box::from(DefaultLocalizer::new(&*LANGUAGE_LOADER_CORE, &Localizations))
 }
