@@ -1,12 +1,14 @@
-use std::collections::{BTreeMap, HashMap};
+use std::collections::BTreeMap;
 use std::path::Path;
 use std::sync::Arc;
 use std::sync::atomic::AtomicBool;
+use std::time::Instant;
 use std::{mem, thread};
 
 use crossbeam_channel::Sender;
 use fun_time::fun_time;
 use humansize::{BINARY, format_size};
+use indexmap::IndexMap;
 use log::debug;
 use rayon::prelude::*;
 
@@ -16,7 +18,7 @@ use crate::common::model::{CheckingMethod, FileEntry, HashType, ToolType, WorkCo
 use crate::common::progress_data::{CurrentStage, ProgressData};
 use crate::common::progress_stop_handler::{check_if_stop_received, prepare_thread_handler_common};
 use crate::common::tool_data::{CommonData, CommonToolData};
-use crate::common::traits::*;
+use crate::common::traits::ResultEntry;
 use crate::tools::duplicate::{
     DuplicateEntry, DuplicateFinder, DuplicateFinderParameters, Info, PREHASHING_BUFFER_SIZE, THREAD_BUFFER, filter_hard_links, hash_calculation, hash_calculation_limit,
 };
@@ -44,7 +46,7 @@ impl DuplicateFinder {
             |fe: &FileEntry| {
                 fe.path
                     .file_name()
-                    .unwrap_or_else(|| panic!("Found invalid file_name \"{}\"", fe.path.to_string_lossy()))
+                    .unwrap_or_else(|| panic!("Found invalid file_name \"{}\" (cannot panic, because it is always normal file)", fe.path.to_string_lossy()))
                     .to_string_lossy()
                     .to_string()
             }
@@ -52,7 +54,7 @@ impl DuplicateFinder {
             |fe: &FileEntry| {
                 fe.path
                     .file_name()
-                    .unwrap_or_else(|| panic!("Found invalid file_name \"{}\"", fe.path.to_string_lossy()))
+                    .unwrap_or_else(|| panic!("Found invalid file_name \"{}\" (cannot panic, because it is always normal file)", fe.path.to_string_lossy()))
                     .to_string_lossy()
                     .to_lowercase()
             }
@@ -133,7 +135,7 @@ impl DuplicateFinder {
                     fe.size,
                     fe.path
                         .file_name()
-                        .unwrap_or_else(|| panic!("Found invalid file_name \"{}\"", fe.path.to_string_lossy()))
+                        .unwrap_or_else(|| panic!("Found invalid file_name \"{}\" (cannot panic, because it is always normal file)", fe.path.to_string_lossy()))
                         .to_string_lossy()
                         .to_string(),
                 )
@@ -144,7 +146,7 @@ impl DuplicateFinder {
                     fe.size,
                     fe.path
                         .file_name()
-                        .unwrap_or_else(|| panic!("Found invalid file_name \"{}\"", fe.path.to_string_lossy()))
+                        .unwrap_or_else(|| panic!("Found invalid file_name \"{}\" (cannot panic, because it is always normal file)", fe.path.to_string_lossy()))
                         .to_string_lossy()
                         .to_lowercase(),
                 )
@@ -237,7 +239,14 @@ impl DuplicateFinder {
                 self.common_data.text_messages.warnings.extend(warnings);
 
                 let grouped_file_entries: Vec<(u64, Vec<FileEntry>)> = grouped_file_entries.into_iter().collect();
-                let rayon_max_len = if self.get_params().ignore_hard_links { 3 } else { 100 };
+                let rayon_max_len = if self.get_hide_hard_links() { 3 } else { 100 };
+
+                let start_time = Instant::now();
+                // We only gather files with more than 1 entry, because only this will be later used
+                let initial_size = grouped_file_entries
+                    .iter()
+                    .map(|(_size, vec)| if vec.len() > 1 { vec.len() as u64 } else { 0 })
+                    .sum::<u64>();
                 self.files_with_identical_size = grouped_file_entries
                     .into_par_iter()
                     .with_max_len(rayon_max_len)
@@ -246,7 +255,7 @@ impl DuplicateFinder {
                             return None;
                         }
 
-                        let vector = if self.get_params().ignore_hard_links { filter_hard_links(&vec) } else { vec };
+                        let vector = if self.get_hide_hard_links() { filter_hard_links(vec) } else { vec };
 
                         if vector.len() > 1 {
                             Some((size, vector.into_iter().map(FileEntry::into_duplicate_entry).collect()))
@@ -255,6 +264,14 @@ impl DuplicateFinder {
                         }
                     })
                     .collect();
+                let filtered_size = self.files_with_identical_size.values().map(|v| v.len() as u64).sum::<u64>();
+                debug!(
+                    "check_file_size - filtered hard links in {:?}, removed {} hardlinks ({} -> {})",
+                    start_time.elapsed(),
+                    initial_size - filtered_size,
+                    initial_size,
+                    filtered_size
+                );
 
                 self.filter_reference_folders_by_size();
                 self.calculate_size_stats();
@@ -325,7 +342,7 @@ impl DuplicateFinder {
 
         if self.get_params().use_prehash_cache {
             let (messages, loaded_items) = load_cache_from_file_generalized_by_size::<DuplicateEntry>(
-                &get_duplicate_cache_file(&self.get_params().hash_type, true),
+                &get_duplicate_cache_file(self.get_params().hash_type, true),
                 self.get_delete_outdated_cache(),
                 &self.files_with_identical_size,
             );
@@ -359,7 +376,7 @@ impl DuplicateFinder {
             for (size, vec_file_entry) in loaded_hash_map {
                 if size >= self.get_params().minimal_prehash_cache_file_size {
                     for file_entry in vec_file_entry {
-                        save_cache_to_hashmap.insert(file_entry.path.to_string_lossy().to_string(), file_entry.clone());
+                        save_cache_to_hashmap.insert(file_entry.path.to_string_lossy().to_string(), file_entry);
                     }
                 }
             }
@@ -368,14 +385,14 @@ impl DuplicateFinder {
                 if size >= self.get_params().minimal_prehash_cache_file_size {
                     for vec_file_entry in hash_map.into_values() {
                         for file_entry in vec_file_entry {
-                            save_cache_to_hashmap.insert(file_entry.path.to_string_lossy().to_string(), file_entry.clone());
+                            save_cache_to_hashmap.insert(file_entry.path.to_string_lossy().to_string(), file_entry);
                         }
                     }
                 }
             }
 
             let messages = save_cache_to_file_generalized(
-                &get_duplicate_cache_file(&self.get_params().hash_type, true),
+                &get_duplicate_cache_file(self.get_params().hash_type, true),
                 &save_cache_to_hashmap,
                 self.common_data.save_also_as_json,
                 self.get_params().minimal_prehash_cache_file_size,
@@ -419,7 +436,7 @@ impl DuplicateFinder {
         let non_cached_files_to_check: Vec<(u64, Vec<DuplicateEntry>)> = non_cached_files_to_check.into_iter().collect();
 
         debug!("Starting calculating prehash");
-        #[allow(clippy::type_complexity)]
+        #[expect(clippy::type_complexity)]
         let pre_hash_results: Vec<(u64, BTreeMap<String, Vec<DuplicateEntry>>, Vec<String>)> = non_cached_files_to_check
             .into_par_iter()
             .with_max_len(3) // Vectors and BTreeMaps for really big inputs, leave some jobs to 0 thread, to avoid that I minimized max tasks for each thread to 3, which improved performance
@@ -497,12 +514,12 @@ impl DuplicateFinder {
         for (size, mut vec_file_entry) in used_map {
             if let Some(cached_vec_file_entry) = loaded_hash_map.get(&size) {
                 // TODO maybe hashmap is not needed when using < 4 elements
-                let mut cached_path_entries: HashMap<&Path, DuplicateEntry> = HashMap::new();
+                let mut cached_path_entries: IndexMap<&Path, DuplicateEntry> = IndexMap::new();
                 for file_entry in cached_vec_file_entry {
                     cached_path_entries.insert(&file_entry.path, file_entry.clone());
                 }
                 for file_entry in vec_file_entry {
-                    if let Some(cached_file_entry) = cached_path_entries.remove(file_entry.path.as_path()) {
+                    if let Some(cached_file_entry) = cached_path_entries.swap_remove(file_entry.path.as_path()) {
                         records_already_cached.entry(size).or_default().push(cached_file_entry);
                     } else {
                         non_cached_files_to_check.entry(size).or_default().push(file_entry);
@@ -533,7 +550,7 @@ impl DuplicateFinder {
         if self.common_data.use_cache {
             debug!("full_hashing_load_cache_at_start - using cache");
             let (messages, loaded_items) = load_cache_from_file_generalized_by_size::<DuplicateEntry>(
-                &get_duplicate_cache_file(&self.get_params().hash_type, false),
+                &get_duplicate_cache_file(self.get_params().hash_type, false),
                 self.get_delete_outdated_cache(),
                 &pre_checked_map,
             );
@@ -599,7 +616,7 @@ impl DuplicateFinder {
         }
 
         let messages = save_cache_to_file_generalized(
-            &get_duplicate_cache_file(&self.get_params().hash_type, false),
+            &get_duplicate_cache_file(self.get_params().hash_type, false),
             &all_results,
             self.common_data.save_also_as_json,
             self.get_params().minimal_cache_file_size,
@@ -665,7 +682,7 @@ impl DuplicateFinder {
                                 }
                             }
                             Err(s) => errors.push(s),
-                        };
+                        }
                         progress_handler.increase_items(1);
                     }
                     Some(())
@@ -725,6 +742,7 @@ impl DuplicateFinder {
                     }
                 })
                 .collect::<Vec<Vec<(DuplicateEntry, Vec<DuplicateEntry>)>>>();
+            #[expect(clippy::indexing_slicing)] // Safe, because here, empty vectors cannot exist
             for vec_of_vec in vec {
                 self.files_with_identical_hashes_referenced.insert(vec_of_vec[0].0.size, vec_of_vec);
             }
@@ -772,7 +790,7 @@ impl DuplicateFinder {
     }
 }
 
-pub fn get_duplicate_cache_file(type_of_hash: &HashType, is_prehash: bool) -> String {
+pub fn get_duplicate_cache_file(type_of_hash: HashType, is_prehash: bool) -> String {
     let prehash_str = if is_prehash { "_prehash" } else { "" };
     format!("cache_duplicates_{type_of_hash:?}{prehash_str}_{CACHE_DUPLICATE_VERSION}.bin")
 }
