@@ -2,6 +2,7 @@ use std::io::prelude::*;
 use std::io::{self};
 use std::sync::Arc;
 use std::sync::atomic::AtomicBool;
+use std::time::Instant;
 
 use crossbeam_channel::Sender;
 use fun_time::fun_time;
@@ -10,7 +11,7 @@ use humansize::{BINARY, format_size};
 use crate::common::model::{CheckingMethod, WorkContinueStatus};
 use crate::common::progress_data::ProgressData;
 use crate::common::tool_data::{CommonData, CommonToolData, DeleteMethod};
-use crate::common::traits::*;
+use crate::common::traits::{AllTraits, DebugPrint, DeletingItems, PrintResults, Search};
 use crate::tools::duplicate::{DuplicateFinder, DuplicateFinderParameters, Info};
 
 impl AllTraits for DuplicateFinder {}
@@ -36,52 +37,60 @@ impl DeletingItems for DuplicateFinder {
 impl Search for DuplicateFinder {
     #[fun_time(message = "find_duplicates", level = "info")]
     fn search(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>) {
-        self.prepare_items();
-        self.common_data.use_reference_folders = !self.common_data.directories.reference_directories.is_empty();
+        let start_time = Instant::now();
 
-        match self.get_params().check_method {
-            CheckingMethod::Name => {
-                self.common_data.stopped_search = self.check_files_name(stop_flag, progress_sender) == WorkContinueStatus::Stop;
-                if self.common_data.stopped_search {
-                    return;
-                }
+        let () = (|| {
+            if self.prepare_items(None).is_err() {
+                return;
             }
-            CheckingMethod::SizeName => {
-                self.common_data.stopped_search = self.check_files_size_name(stop_flag, progress_sender) == WorkContinueStatus::Stop;
-                if self.common_data.stopped_search {
-                    return;
+            self.common_data.use_reference_folders = !self.common_data.directories.reference_directories.is_empty() || !self.common_data.directories.reference_files.is_empty();
+
+            match self.get_params().check_method {
+                CheckingMethod::Name => {
+                    self.common_data.stopped_search = self.check_files_name(stop_flag, progress_sender) == WorkContinueStatus::Stop;
+                    if self.common_data.stopped_search {
+                        return;
+                    }
                 }
+                CheckingMethod::SizeName => {
+                    self.common_data.stopped_search = self.check_files_size_name(stop_flag, progress_sender) == WorkContinueStatus::Stop;
+                    if self.common_data.stopped_search {
+                        return;
+                    }
+                }
+                CheckingMethod::Size => {
+                    self.common_data.stopped_search = self.check_files_size(stop_flag, progress_sender) == WorkContinueStatus::Stop;
+                    if self.common_data.stopped_search {
+                        return;
+                    }
+                }
+                CheckingMethod::Hash => {
+                    self.common_data.stopped_search = self.check_files_size(stop_flag, progress_sender) == WorkContinueStatus::Stop;
+                    if self.common_data.stopped_search {
+                        return;
+                    }
+                    self.common_data.stopped_search = self.check_files_hash(stop_flag, progress_sender) == WorkContinueStatus::Stop;
+                    if self.common_data.stopped_search {
+                        return;
+                    }
+                }
+                _ => panic!(),
             }
-            CheckingMethod::Size => {
-                self.common_data.stopped_search = self.check_files_size(stop_flag, progress_sender) == WorkContinueStatus::Stop;
-                if self.common_data.stopped_search {
-                    return;
-                }
+            if self.delete_files(stop_flag, progress_sender) == WorkContinueStatus::Stop {
+                self.common_data.stopped_search = true;
             }
-            CheckingMethod::Hash => {
-                self.common_data.stopped_search = self.check_files_size(stop_flag, progress_sender) == WorkContinueStatus::Stop;
-                if self.common_data.stopped_search {
-                    return;
-                }
-                self.common_data.stopped_search = self.check_files_hash(stop_flag, progress_sender) == WorkContinueStatus::Stop;
-                if self.common_data.stopped_search {
-                    return;
-                }
-            }
-            _ => panic!(),
-        }
-        if self.delete_files(stop_flag, progress_sender) == WorkContinueStatus::Stop {
-            self.common_data.stopped_search = true;
-            return;
-        };
+        })();
+
+        self.information.scanning_time = start_time.elapsed();
+
         self.debug_print();
     }
 }
 
 impl DebugPrint for DuplicateFinder {
-    #[allow(clippy::print_stdout)]
+    #[expect(clippy::print_stdout)]
     fn debug_print(&self) {
-        if !cfg!(debug_assertions) {
+        if !cfg!(debug_assertions) || cfg!(test) {
             return;
         }
         println!("---------------DEBUG PRINT---------------");
@@ -111,7 +120,7 @@ impl DebugPrint for DuplicateFinder {
         println!("### Other");
 
         println!("Files list size - {}", self.files_with_identical_size.len());
-        println!("Hashed Files list size - {}", self.files_with_identical_hashes.len());
+        println!("Hashed files list size - {}", self.files_with_identical_hashes.len());
         println!("Files with identical names - {}", self.files_with_identical_names.len());
         println!("Files with identical size names - {}", self.files_with_identical_size_names.len());
         println!("Files with identical names referenced - {}", self.files_with_identical_names_referenced.len());
@@ -126,14 +135,7 @@ impl DebugPrint for DuplicateFinder {
 
 impl PrintResults for DuplicateFinder {
     fn write_results<T: Write>(&self, writer: &mut T) -> io::Result<()> {
-        writeln!(
-            writer,
-            "Results of searching {:?} (reference directories {:?}) with excluded directories {:?} and excluded items {:?}",
-            self.common_data.directories.included_directories,
-            self.common_data.directories.reference_directories,
-            self.common_data.directories.excluded_directories,
-            self.common_data.excluded_items.get_excluded_items()
-        )?;
+        self.write_base_search_paths(writer)?;
 
         match self.get_params().check_method {
             CheckingMethod::Name => {
@@ -166,7 +168,7 @@ impl PrintResults for DuplicateFinder {
                     )?;
                     for (name, (file_entry, vector)) in self.files_with_identical_names_referenced.iter().rev() {
                         writeln!(writer, "Name - {} - {} files ", name, vector.len())?;
-                        writeln!(writer, "Reference file - {:?}", file_entry.path)?;
+                        writeln!(writer, "Reference file - \"{}\"", file_entry.path.to_string_lossy())?;
                         for j in vector {
                             writeln!(writer, "\"{}\"", j.path.to_string_lossy())?;
                         }
@@ -206,7 +208,7 @@ impl PrintResults for DuplicateFinder {
                     )?;
                     for ((size, name), (file_entry, vector)) in self.files_with_identical_size_names_referenced.iter().rev() {
                         writeln!(writer, "Name - {}, {} - {} files ", name, format_size(*size, BINARY), vector.len())?;
-                        writeln!(writer, "Reference file - {:?}", file_entry.path)?;
+                        writeln!(writer, "Reference file - \"{}\"", file_entry.path.to_string_lossy())?;
                         for j in vector {
                             writeln!(writer, "\"{}\"", j.path.to_string_lossy())?;
                         }
@@ -249,7 +251,7 @@ impl PrintResults for DuplicateFinder {
                     )?;
                     for (size, (file_entry, vector)) in self.files_with_identical_size_referenced.iter().rev() {
                         writeln!(writer, "\n---- Size {} ({}) - {} files", format_size(*size, BINARY), size, vector.len())?;
-                        writeln!(writer, "Reference file - {:?}", file_entry.path)?;
+                        writeln!(writer, "Reference file - \"{}\"", file_entry.path.to_string_lossy())?;
                         for file_entry in vector {
                             writeln!(writer, "\"{}\"", file_entry.path.to_string_lossy())?;
                         }
@@ -339,7 +341,7 @@ impl CommonData for DuplicateFinder {
     type Parameters = DuplicateFinderParameters;
 
     fn get_information(&self) -> Self::Info {
-        self.information.clone()
+        self.information
     }
     fn get_params(&self) -> Self::Parameters {
         self.params.clone()
@@ -353,7 +355,7 @@ impl CommonData for DuplicateFinder {
     fn get_check_method(&self) -> CheckingMethod {
         self.get_params().check_method
     }
-    fn found_any_broken_files(&self) -> bool {
+    fn found_any_items(&self) -> bool {
         self.get_information().number_of_duplicated_files_by_hash > 0
             || self.get_information().number_of_duplicated_files_by_name > 0
             || self.get_information().number_of_duplicated_files_by_size > 0
