@@ -1,12 +1,17 @@
 pub mod core;
+#[cfg(test)]
+mod tests;
 pub mod traits;
 
+use std::time::Duration;
+
 use crate::common::model::FileEntry;
 use crate::common::tool_data::CommonToolData;
 
-#[derive(Default, Clone)]
+#[derive(Default, Clone, Copy)]
 pub struct Info {
     pub number_of_empty_files: usize,
+    pub scanning_time: Duration,
 }
 
 pub struct EmptyFiles {
@@ -26,7 +31,7 @@ impl EmptyFiles {
         &self.empty_files
     }
 
-    pub const fn get_information(&self) -> &Info {
-        &self.information
+    pub const fn get_information(&self) -> Info {
+        self.information
     }
 }
