@@ -1,8 +1,12 @@
 pub mod core;
+#[cfg(test)]
+mod tests;
 pub mod traits;
 
-use std::collections::HashMap;
 use std::path::{Path, PathBuf};
+use std::time::Duration;
+
+use indexmap::IndexMap;
 
 use crate::common::tool_data::CommonToolData;
 use crate::common::traits::ResultEntry;
@@ -33,7 +37,7 @@ impl ResultEntry for FolderEntry {
 pub struct EmptyFolder {
     common_data: CommonToolData,
     information: Info,
-    empty_folder_list: HashMap<String, FolderEntry>, // Path, FolderEntry
+    empty_folder_list: IndexMap<String, FolderEntry>, // Path, FolderEntry
 }
 
 /// Enum with values which show if folder is empty.
@@ -44,9 +48,10 @@ pub(crate) enum FolderEmptiness {
     Maybe,
 }
 
-#[derive(Default, Clone)]
+#[derive(Default, Clone, Copy)]
 pub struct Info {
     pub number_of_empty_folders: usize,
+    pub scanning_time: Duration,
 }
 
 impl Default for EmptyFolder {
