@@ -2,11 +2,12 @@ pub mod core;
 pub mod traits;
 
 use std::path::{Path, PathBuf};
+use std::time::Duration;
 
 use serde::Serialize;
 
 use crate::common::tool_data::CommonToolData;
-use crate::common::traits::*;
+use crate::common::traits::ResultEntry;
 
 const TEMP_EXTENSIONS: &[&str] = &[
     "#",
@@ -43,9 +44,10 @@ impl ResultEntry for TemporaryFileEntry {
     }
 }
 
-#[derive(Default, Clone)]
+#[derive(Default, Clone, Copy)]
 pub struct Info {
     pub number_of_temporary_files: usize,
+    pub scanning_time: Duration,
 }
 
 pub struct Temporary {
@@ -65,7 +67,7 @@ impl Temporary {
         &self.temporary_files
     }
 
-    pub const fn get_information(&self) -> &Info {
-        &self.information
+    pub const fn get_information(&self) -> Info {
+        self.information
     }
 }
