@@ -4,16 +4,18 @@ use std::rc::Rc;
 use czkawka_core::common::image::get_dynamic_image_from_path;
 use gdk4::gdk_pixbuf::{InterpType, Pixbuf};
 use gtk4::prelude::*;
-use gtk4::{Align, CheckButton, Image, ListStore, Orientation, ScrolledWindow, TreeIter, TreeModel, TreePath, TreeSelection, Widget};
+use gtk4::{Align, CheckButton, Image, Orientation, Picture, ScrolledWindow, TreeIter, TreeModel, TreePath, Widget};
 use image::DynamicImage;
 use log::error;
 
 use crate::flg;
+use crate::gtk_traits::WidgetTraits;
+use crate::gui_structs::common_tree_view::SubView;
 use crate::gui_structs::gui_data::GuiData;
-use crate::help_functions::{
-    count_number_of_groups, get_all_direct_children, get_full_name_from_path_name, get_max_file_name, get_pixbuf_from_dynamic_image, resize_pixbuf_dimension,
-};
-use crate::notebook_info::{NOTEBOOKS_INFO, NotebookObject};
+use crate::help_functions::{get_full_name_from_path_name, get_max_file_name};
+use crate::helpers::image_operations::{get_pixbuf_from_dynamic_image, resize_pixbuf_dimension};
+use crate::helpers::list_store_operations::count_number_of_groups;
+use crate::notebook_info::NotebookObject;
 
 const BIG_PREVIEW_SIZE: i32 = 600;
 const SMALL_PREVIEW_SIZE: i32 = 130;
@@ -21,8 +23,7 @@ const SMALL_PREVIEW_SIZE: i32 = 130;
 pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let button_compare = gui_data.bottom_buttons.buttons_compare.clone();
     let window_compare = gui_data.compare_images.window_compare.clone();
-    let notebook_main = gui_data.main_notebook.notebook_main.clone();
-    let main_tree_views = gui_data.main_notebook.get_main_tree_views();
+    let common_tree_views = gui_data.main_notebook.common_tree_views.clone();
     let scrolled_window_compare_choose_images = gui_data.compare_images.scrolled_window_compare_choose_images.clone();
 
     let label_group_info = gui_data.compare_images.label_group_info.clone();
@@ -47,25 +48,23 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     window_compare.set_default_size(700, 700);
 
     button_compare.connect_clicked(move |_| {
-        let nb_number = notebook_main.current_page().expect("Current page not set");
-        let tree_view = &main_tree_views[nb_number as usize];
-        let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().expect("Missing tree_view model");
+        let subview = common_tree_views.get_current_subview();
+        let model = subview.tree_view.model().expect("Missing tree_view model");
 
-        let group_number = count_number_of_groups(tree_view, nb_object.column_header.expect("Missing column_header"));
+        let group_number = count_number_of_groups(subview);
 
         if group_number == 0 {
             return;
         }
 
         // Check selected items
-        let (current_group, tree_path) = get_current_group_and_iter_from_selection(&model, &tree_view.selection(), nb_object.column_header.expect("Missing column_header"));
+        let (current_group, tree_path) = get_current_group_and_iter_from_selection(subview);
 
         *shared_current_of_groups.borrow_mut() = current_group;
         *shared_numbers_of_groups.borrow_mut() = group_number;
 
         populate_groups_at_start(
-            nb_object,
+            &subview.nb_object,
             &model,
             &shared_current_path,
             tree_path,
@@ -84,7 +83,7 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
             &check_button_settings_use_rust_preview,
         );
 
-        window_compare.show();
+        window_compare.set_visible(true);
     });
 
     let shared_image_cache = gui_data.compare_images.shared_image_cache.clone();
@@ -96,22 +95,20 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_left = gui_data.compare_images.image_compare_left.clone();
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
     window_compare.connect_close_request(move |window_compare| {
-        window_compare.hide();
+        window_compare.set_visible(false);
         *shared_image_cache.borrow_mut() = Vec::new();
         *shared_current_path.borrow_mut() = None;
         *shared_current_of_groups.borrow_mut() = 0;
         *shared_numbers_of_groups.borrow_mut() = 0;
         *shared_using_for_preview.borrow_mut() = (None, None);
-        image_compare_left.set_from_pixbuf(None);
-        image_compare_right.set_from_pixbuf(None);
+        image_compare_left.set_pixbuf(None);
+        image_compare_right.set_pixbuf(None);
         glib::Propagation::Stop
     });
 
     let button_go_previous_compare_group = gui_data.compare_images.button_go_previous_compare_group.clone();
     let button_go_next_compare_group = gui_data.compare_images.button_go_next_compare_group.clone();
     let label_group_info = gui_data.compare_images.label_group_info.clone();
-    let notebook_main = gui_data.main_notebook.notebook_main.clone();
-    let main_tree_views = gui_data.main_notebook.get_main_tree_views();
     let scrolled_window_compare_choose_images = gui_data.compare_images.scrolled_window_compare_choose_images.clone();
 
     let check_button_left_preview_text = gui_data.compare_images.check_button_left_preview_text.clone();
@@ -127,12 +124,11 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
 
     let check_button_settings_use_rust_preview = gui_data.settings.check_button_settings_use_rust_preview.clone();
+    let common_tree_views = gui_data.main_notebook.common_tree_views.clone();
 
     button_go_previous_compare_group.connect_clicked(move |button_go_previous_compare_group| {
-        let nb_number = notebook_main.current_page().expect("Current page not set");
-        let tree_view = &main_tree_views[nb_number as usize];
-        let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().expect("Missing tree_view model");
+        let sv = common_tree_views.get_current_subview();
+        let model = sv.get_tree_model();
 
         *shared_current_of_groups.borrow_mut() -= 1;
 
@@ -142,12 +138,12 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
         let tree_path = move_iter(
             &model,
             shared_current_path.borrow().as_ref().expect("Missing current path"),
-            nb_object.column_header.expect("Missing column_header"),
+            sv.nb_object.column_header.expect("Missing column_header"),
             false,
         );
 
         populate_groups_at_start(
-            nb_object,
+            &sv.nb_object,
             &model,
             &shared_current_path,
             tree_path,
@@ -170,8 +166,6 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let button_go_previous_compare_group = gui_data.compare_images.button_go_previous_compare_group.clone();
     let button_go_next_compare_group = gui_data.compare_images.button_go_next_compare_group.clone();
     let label_group_info = gui_data.compare_images.label_group_info.clone();
-    let notebook_main = gui_data.main_notebook.notebook_main.clone();
-    let main_tree_views = gui_data.main_notebook.get_main_tree_views();
     let scrolled_window_compare_choose_images = gui_data.compare_images.scrolled_window_compare_choose_images.clone();
 
     let check_button_left_preview_text = gui_data.compare_images.check_button_left_preview_text.clone();
@@ -187,12 +181,11 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
 
     let check_button_settings_use_rust_preview = gui_data.settings.check_button_settings_use_rust_preview.clone();
+    let common_tree_views = gui_data.main_notebook.common_tree_views.clone();
 
     button_go_next_compare_group.connect_clicked(move |button_go_next_compare_group| {
-        let nb_number = notebook_main.current_page().expect("Current page not set");
-        let tree_view = &main_tree_views[nb_number as usize];
-        let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().expect("Missing tree_view model");
+        let sv = common_tree_views.get_current_subview();
+        let model = sv.get_tree_model();
 
         *shared_current_of_groups.borrow_mut() += 1;
 
@@ -202,12 +195,12 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
         let tree_path = move_iter(
             &model,
             shared_current_path.borrow().as_ref().expect("Missing current path"),
-            nb_object.column_header.expect("Missing column_header"),
+            sv.nb_object.column_header.expect("Missing column_header"),
             true,
         );
 
         populate_groups_at_start(
-            nb_object,
+            &sv.nb_object,
             &model,
             &shared_current_path,
             tree_path,
@@ -227,20 +220,22 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
         );
     });
 
+    let button_replace_group = gui_data.compare_images.button_replace_group.clone();
+    let image_compare_right = gui_data.compare_images.image_compare_right.clone();
+    let image_compare_left = gui_data.compare_images.image_compare_left.clone();
+    button_replace_group.connect_clicked(move |_| {
+        let tmp = image_compare_left.paintable();
+        image_compare_left.set_paintable(image_compare_right.paintable().as_ref());
+        image_compare_right.set_paintable(tmp.as_ref());
+    });
+
     let check_button_left_preview_text = gui_data.compare_images.check_button_left_preview_text.clone();
     let shared_using_for_preview = gui_data.compare_images.shared_using_for_preview.clone();
-    let notebook_main = gui_data.main_notebook.notebook_main.clone();
     let shared_current_path = gui_data.compare_images.shared_current_path.clone();
-    let main_tree_views = gui_data.main_notebook.get_main_tree_views();
+    let common_tree_views = gui_data.main_notebook.common_tree_views.clone();
     check_button_left_preview_text.connect_toggled(move |check_button_left_preview_text| {
-        let nb_number = notebook_main.current_page().expect("Current page not set");
-        let tree_view = &main_tree_views[nb_number as usize];
-        let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view
-            .model()
-            .expect("Missing tree_view model")
-            .downcast::<ListStore>()
-            .expect("Failed to downcast to ListStore");
+        let sv = common_tree_views.get_current_subview();
+        let model = sv.get_model();
 
         let main_tree_path = shared_current_path.borrow().as_ref().expect("Missing current path").clone();
         let this_tree_path = shared_using_for_preview.borrow().0.clone().expect("Missing left preview path");
@@ -252,7 +247,7 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
         let is_active = check_button_left_preview_text.is_active();
         model.set_value(
             &model.iter(&this_tree_path).expect("Using invalid tree_path"),
-            nb_object.column_selection as u32,
+            sv.nb_object.column_selection as u32,
             &is_active.to_value(),
         );
     });
@@ -260,18 +255,11 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let check_button_right_preview_text = gui_data.compare_images.check_button_right_preview_text.clone();
     let shared_using_for_preview = gui_data.compare_images.shared_using_for_preview.clone();
     let shared_current_path = gui_data.compare_images.shared_current_path.clone();
-    let notebook_main = gui_data.main_notebook.notebook_main.clone();
-    let main_tree_views = gui_data.main_notebook.get_main_tree_views();
+    let common_tree_views = gui_data.main_notebook.common_tree_views.clone();
 
     check_button_right_preview_text.connect_toggled(move |check_button_right_preview_text| {
-        let nb_number = notebook_main.current_page().expect("Current page not set");
-        let tree_view = &main_tree_views[nb_number as usize];
-        let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view
-            .model()
-            .expect("Missing tree_view model")
-            .downcast::<ListStore>()
-            .expect("Failed to downcast to ListStore");
+        let sv = common_tree_views.get_current_subview();
+        let model = sv.get_model();
 
         let main_tree_path = shared_current_path.borrow().as_ref().expect("Missing current path").clone();
         let this_tree_path = shared_using_for_preview.borrow().1.clone().expect("Missing right preview path");
@@ -283,7 +271,7 @@ pub(crate) fn connect_button_compare(gui_data: &GuiData) {
         let is_active = check_button_right_preview_text.is_active();
         model.set_value(
             &model.iter(&this_tree_path).expect("Using invalid tree_path"),
-            nb_object.column_selection as u32,
+            sv.nb_object.column_selection as u32,
             &is_active.to_value(),
         );
     });
@@ -294,8 +282,8 @@ fn populate_groups_at_start(
     model: &TreeModel,
     shared_current_path: &Rc<RefCell<Option<TreePath>>>,
     tree_path: TreePath,
-    image_compare_left: &Image,
-    image_compare_right: &Image,
+    image_compare_left: &Picture,
+    image_compare_right: &Picture,
     current_group: u32,
     group_number: u32,
     check_button_left_preview_text: &CheckButton,
@@ -365,13 +353,13 @@ fn populate_groups_at_start(
     *shared_image_cache.borrow_mut() = cache_all_images.clone();
 
     let mut found = false;
-    for i in get_all_direct_children(
-        &scrolled_window_compare_choose_images
-            .child()
-            .expect("Failed to get child of scrolled_window_compare_choose_images")
-            .downcast::<gtk4::Viewport>()
-            .expect("Failed to downcast to Viewport"),
-    ) {
+    for i in scrolled_window_compare_choose_images
+        .child()
+        .expect("Failed to get child of scrolled_window_compare_choose_images")
+        .downcast::<gtk4::Viewport>()
+        .expect("Failed to downcast to Viewport")
+        .get_all_direct_children()
+    {
         if i.widget_name() == "all_box" {
             let gtk_box = i.downcast::<gtk4::Box>().expect("Failed to downcast to Box");
             update_bottom_buttons(&gtk_box, shared_using_for_preview, shared_image_cache);
@@ -395,24 +383,17 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>,
         let small_img = Image::new();
         let big_img = Image::new();
 
-        let mut pixbuf = get_pixbuf_from_dynamic_image(&DynamicImage::new_rgb8(1, 1)).expect("Failed to create pixbuf");
+        let mut pixbuf = get_pixbuf_from_dynamic_image(DynamicImage::new_rgb8(1, 1)).expect("Failed to create pixbuf");
 
         if use_rust_loader {
-            match get_dynamic_image_from_path(&full_path) {
+            match get_dynamic_image_from_path(&full_path).and_then(get_pixbuf_from_dynamic_image) {
                 Ok(t) => {
-                    match get_pixbuf_from_dynamic_image(&t) {
-                        Ok(t) => {
-                            pixbuf = t;
-                        }
-                        Err(e) => {
-                            error!("Failed to open image {full_path}, reason {e}");
-                        }
-                    };
+                    pixbuf = t;
                 }
                 Err(e) => {
                     error!("Failed to open image {full_path}, reason {e}");
                 }
-            };
+            }
         } else {
             match Pixbuf::from_file(&full_path) {
                 Ok(t) => {
@@ -421,10 +402,10 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>,
                 Err(e) => {
                     error!("Failed to open image {full_path}, reason {e}");
                 }
-            };
+            }
         }
 
-        #[allow(clippy::never_loop)]
+        #[expect(clippy::never_loop)]
         loop {
             let Some(pixbuf_big) = resize_pixbuf_dimension(&pixbuf, (BIG_PREVIEW_SIZE, BIG_PREVIEW_SIZE), InterpType::Bilinear) else {
                 error!("Failed to resize image {full_path}.");
@@ -446,7 +427,7 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>,
 }
 
 fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32, column_path: i32, column_name: i32) -> Vec<(String, String, TreePath)> {
-    let used_iter = model.iter(current_path).expect("Using invalid tree_path");
+    let mut used_iter = model.iter(current_path).expect("Using invalid tree_path");
 
     assert!(model.get::<bool>(&used_iter, column_header));
     let using_reference = !model.get::<String>(&used_iter, column_path).is_empty();
@@ -462,7 +443,7 @@ fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32,
         returned_vector.push((full_name, name, model.path(&used_iter)));
     }
 
-    assert!(model.iter_next(&used_iter), "Found only header!");
+    assert!(model.iter_next(&mut used_iter), "Found only header!");
 
     loop {
         let name = model.get::<String>(&used_iter, column_name);
@@ -472,7 +453,7 @@ fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32,
 
         returned_vector.push((full_name, name, model.path(&used_iter)));
 
-        if !model.iter_next(&used_iter) {
+        if !model.iter_next(&mut used_iter) {
             break;
         }
 
@@ -487,25 +468,23 @@ fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32,
 }
 
 fn move_iter(model: &TreeModel, tree_path: &TreePath, column_header: i32, go_next: bool) -> TreePath {
-    let tree_iter = model.iter(tree_path).expect("Using invalid tree_path");
+    let mut tree_iter = model.iter(tree_path).expect("Using invalid tree_path");
 
     assert!(model.get::<bool>(&tree_iter, column_header));
 
     if go_next {
-        assert!(model.iter_next(&tree_iter), "Found only header!");
+        assert!(model.iter_next(&mut tree_iter), "Found only header!");
     } else {
-        assert!(model.iter_previous(&tree_iter), "Found only header!");
+        assert!(model.iter_previous(&mut tree_iter), "Found only header!");
     }
 
     loop {
         if go_next {
-            if !model.iter_next(&tree_iter) {
-                break;
-            }
-        } else {
-            if !model.iter_previous(&tree_iter) {
+            if !model.iter_next(&mut tree_iter) {
                 break;
             }
+        } else if !model.iter_previous(&mut tree_iter) {
+            break;
         }
 
         if model.get::<bool>(&tree_iter, column_header) {
@@ -518,8 +497,8 @@ fn move_iter(model: &TreeModel, tree_path: &TreePath, column_header: i32, go_nex
 fn populate_similar_scrolled_view(
     scrolled_window: &ScrolledWindow,
     image_cache: &[(String, String, Image, Image, TreePath)],
-    image_compare_left: &Image,
-    image_compare_right: &Image,
+    image_compare_left: &Picture,
+    image_compare_right: &Picture,
     shared_using_for_preview: &Rc<RefCell<(Option<TreePath>, Option<TreePath>)>>,
     shared_image_cache: &Rc<RefCell<Vec<(String, String, Image, Image, TreePath)>>>,
     check_button_left_preview_text: &CheckButton,
@@ -605,7 +584,7 @@ fn populate_similar_scrolled_view(
         all_gtk_box.append(&small_box);
     }
 
-    all_gtk_box.show();
+    all_gtk_box.set_visible(true);
     scrolled_window.set_child(Some(&all_gtk_box));
 }
 
@@ -617,13 +596,13 @@ fn update_bottom_buttons(
     let left_tree_view = shared_using_for_preview.borrow().0.clone().expect("Left tree_view not set");
     let right_tree_view = shared_using_for_preview.borrow().1.clone().expect("Right tree_view not set");
 
-    for (number, i) in get_all_direct_children(all_gtk_box).into_iter().enumerate() {
+    for (number, i) in all_gtk_box.get_all_direct_children().into_iter().enumerate() {
         let cache_tree_path = (*image_cache.borrow())[number].4.clone();
         let is_chosen = cache_tree_path != right_tree_view && cache_tree_path != left_tree_view;
 
         let bx = i.downcast::<gtk4::Box>().expect("Not Box");
         let smaller_bx = bx.first_child().expect("No first child").downcast::<gtk4::Box>().expect("First child is not Box");
-        for items in get_all_direct_children(&smaller_bx) {
+        for items in smaller_bx.get_all_direct_children() {
             if let Ok(btn) = items.downcast::<gtk4::Button>() {
                 btn.set_sensitive(is_chosen);
             }
@@ -631,15 +610,19 @@ fn update_bottom_buttons(
     }
 }
 
-fn get_current_group_and_iter_from_selection(model: &TreeModel, selection: &TreeSelection, column_header: i32) -> (u32, TreePath) {
+fn get_current_group_and_iter_from_selection(sv: &SubView) -> (u32, TreePath) {
     let mut current_group = 1;
     let mut possible_group = 1;
     let mut header_clone: TreeIter;
     let mut possible_header: TreeIter;
 
+    let column_header = sv.nb_object.column_header.expect("Missing column_header");
+    let model = sv.get_tree_model();
+    let selection = sv.get_tree_selection();
+
     let selected_records = selection.selected_rows().0;
 
-    let iter = model.iter_first().expect("Model is no empty, so should have first item"); // Checking that treeview is not empty should be done before
+    let mut iter = model.iter_first().expect("Model is no empty, so should have first item"); // Checking that treeview is not empty should be done before
     header_clone = iter; // if nothing selected, use first group
     possible_header = iter; // if nothing selected, use first group
     assert!(model.get::<bool>(&iter, column_header)); // First element should be header
@@ -647,7 +630,7 @@ fn get_current_group_and_iter_from_selection(model: &TreeModel, selection: &Tree
     if !selected_records.is_empty() {
         let first_selected_record = selected_records[0].clone();
         loop {
-            if !model.iter_next(&iter) {
+            if !model.iter_next(&mut iter) {
                 break;
             }
 
