@@ -24,17 +24,21 @@ use fun_time::fun_time;
 use gtk4::Grid;
 use gtk4::prelude::*;
 
+use crate::gui_structs::common_tree_view::TreeViewListStoreTrait;
+use crate::gui_structs::common_upper_tree_view::UpperTreeViewEnum;
 use crate::gui_structs::gui_data::GuiData;
 use crate::help_combo_box::{
     AUDIO_TYPE_CHECK_METHOD_COMBO_BOX, BIG_FILES_CHECK_METHOD_COMBO_BOX, DUPLICATES_CHECK_METHOD_COMBO_BOX, DUPLICATES_HASH_TYPE_COMBO_BOX, IMAGES_HASH_SIZE_COMBO_BOX,
     IMAGES_HASH_TYPE_COMBO_BOX, IMAGES_RESIZE_ALGORITHM_COMBO_BOX,
 };
-use crate::help_functions::*;
-use crate::notebook_enums::*;
+use crate::help_functions::{get_path_buf_from_vector_of_strings, hide_all_buttons, reset_text_view, set_buttons};
+use crate::helpers::enums::{ColumnsExcludedDirectory, ColumnsIncludedDirectory, Message};
+use crate::helpers::list_store_operations::{check_if_list_store_column_have_all_same_values, get_string_from_list_store};
+use crate::helpers::model_iter::iter_list;
+use crate::notebook_enums::NotebookMainEnum;
 use crate::taskbar_progress::tbp_flags::TBPF_NOPROGRESS;
 use crate::{DEFAULT_MAXIMAL_FILE_SIZE, DEFAULT_MINIMAL_CACHE_SIZE, DEFAULT_MINIMAL_FILE_SIZE, flg};
 
-#[allow(clippy::too_many_arguments)]
 pub(crate) fn connect_button_search(gui_data: &GuiData, result_sender: Sender<Message>, progress_sender: Sender<ProgressData>) {
     let buttons_array = gui_data.bottom_buttons.buttons_array.clone();
     let buttons_search_clone = gui_data.bottom_buttons.buttons_search.clone();
@@ -47,7 +51,11 @@ pub(crate) fn connect_button_search(gui_data: &GuiData, result_sender: Sender<Me
     let stop_flag = gui_data.stop_flag.clone();
     let taskbar_state = gui_data.taskbar_state.clone();
     let text_view_errors = gui_data.text_view_errors.clone();
-    let tree_view_included_directories = gui_data.upper_notebook.tree_view_included_directories.clone();
+    let tree_view_included_directories = gui_data
+        .upper_notebook
+        .common_upper_tree_views
+        .get_tree_view(UpperTreeViewEnum::IncludedDirectories)
+        .clone();
     let window_progress = gui_data.progress_window.window_progress.clone();
     let entry_info = gui_data.entry_info.clone();
     let button_settings = gui_data.header.button_settings.clone();
@@ -58,7 +66,7 @@ pub(crate) fn connect_button_search(gui_data: &GuiData, result_sender: Sender<Me
         let loaded_commons = LoadedCommonItems::load_items(&gui_data);
 
         // Check if user selected all referenced folders
-        let list_store_included_directories = get_list_store(&tree_view_included_directories);
+        let list_store_included_directories = tree_view_included_directories.get_model();
         if check_if_list_store_column_have_all_same_values(&list_store_included_directories, ColumnsIncludedDirectory::ReferenceButton as i32, true) {
             entry_info.set_text(&flg!("selected_all_reference_folders"));
             return;
@@ -88,11 +96,12 @@ pub(crate) fn connect_button_search(gui_data: &GuiData, result_sender: Sender<Me
         // Clear stop flag
         stop_flag.store(false, Ordering::Relaxed);
 
-        label_stage.show();
+        label_stage.set_visible(true);
 
         let progress_sender = progress_sender.clone();
 
-        match to_notebook_main_enum(notebook_main.current_page().expect("Current page not set")) {
+        let current_data = gui_data.main_notebook.common_tree_views.clone();
+        match current_data.get_current_page() {
             NotebookMainEnum::Duplicate => duplicate_search(&gui_data, loaded_commons, stop_flag, result_sender, &grid_progress, progress_sender),
             NotebookMainEnum::EmptyFiles => empty_files_search(&gui_data, loaded_commons, stop_flag, result_sender, &grid_progress, progress_sender),
             NotebookMainEnum::EmptyDirectories => empty_dirs_search(&gui_data, loaded_commons, stop_flag, result_sender, &grid_progress, progress_sender),
@@ -146,8 +155,16 @@ impl LoadedCommonItems {
         let entry_general_maximal_size = gui_data.upper_notebook.entry_general_maximal_size.clone();
         let entry_general_minimal_size = gui_data.upper_notebook.entry_general_minimal_size.clone();
         let entry_settings_cache_file_minimal_size = gui_data.settings.entry_settings_cache_file_minimal_size.clone();
-        let tree_view_excluded_directories = gui_data.upper_notebook.tree_view_excluded_directories.clone();
-        let tree_view_included_directories = gui_data.upper_notebook.tree_view_included_directories.clone();
+        let tree_view_excluded_directories = gui_data
+            .upper_notebook
+            .common_upper_tree_views
+            .get_tree_view(UpperTreeViewEnum::ExcludedDirectories)
+            .clone();
+        let tree_view_included_directories = gui_data
+            .upper_notebook
+            .common_upper_tree_views
+            .get_tree_view(UpperTreeViewEnum::IncludedDirectories)
+            .clone();
         let check_button_settings_save_also_json = gui_data.settings.check_button_settings_save_also_json.clone();
 
         let included_directories = get_path_buf_from_vector_of_strings(&get_string_from_list_store(&tree_view_included_directories, ColumnsIncludedDirectory::Path as i32, None));
@@ -170,11 +187,14 @@ impl LoadedCommonItems {
             .parse::<u64>()
             .unwrap_or_else(|_| DEFAULT_MINIMAL_CACHE_SIZE.parse::<u64>().expect("Failed to parse minimal_cache_file_size"));
 
-        let minimal_file_size = entry_general_minimal_size
-            .text()
-            .as_str()
-            .parse::<u64>()
-            .unwrap_or_else(|_| DEFAULT_MINIMAL_FILE_SIZE.parse::<u64>().expect("Failed to parse minimal_file_size"));
+        let minimal_file_size_txt = entry_general_minimal_size.text().trim().to_string();
+        let minimal_file_size = if minimal_file_size_txt.is_empty() {
+            0u64
+        } else {
+            minimal_file_size_txt
+                .parse::<u64>()
+                .unwrap_or_else(|_| DEFAULT_MINIMAL_FILE_SIZE.parse::<u64>().expect("Failed to parse minimal_file_size"))
+        };
         let maximal_file_size = entry_general_maximal_size
             .text()
             .as_str()
@@ -209,7 +229,7 @@ fn duplicate_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
     let combo_box_duplicate_check_method = gui_data.main_notebook.combo_box_duplicate_check_method.clone();
     let combo_box_duplicate_hash_type = gui_data.main_notebook.combo_box_duplicate_hash_type.clone();
@@ -218,10 +238,9 @@ fn duplicate_search(
     let check_button_settings_duplicates_delete_outdated_cache = gui_data.settings.check_button_settings_duplicates_delete_outdated_cache.clone();
     let entry_settings_prehash_cache_file_minimal_size = gui_data.settings.entry_settings_prehash_cache_file_minimal_size.clone();
     let image_preview_duplicates = gui_data.main_notebook.image_preview_duplicates.clone();
-    let tree_view_duplicate_finder = gui_data.main_notebook.tree_view_duplicate_finder.clone();
 
-    image_preview_duplicates.hide();
-    clean_tree_view(&tree_view_duplicate_finder);
+    image_preview_duplicates.set_visible(false);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     let check_method_index = combo_box_duplicate_check_method.active().expect("Failed to get active search") as usize;
     let check_method = DUPLICATES_CHECK_METHOD_COMBO_BOX[check_method_index].check_method;
@@ -243,7 +262,6 @@ fn duplicate_search(
             let params = DuplicateFinderParameters::new(
                 check_method,
                 hash_type,
-                loaded_commons.hide_hard_links,
                 use_prehash_cache,
                 loaded_commons.minimal_cache_file_size,
                 minimal_prehash_cache_file_size,
@@ -267,10 +285,9 @@ fn empty_files_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.hide();
+    grid_progress.set_visible(false);
 
-    let tree_view_empty_files_finder = gui_data.main_notebook.tree_view_empty_files_finder.clone();
-    clean_tree_view(&tree_view_empty_files_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
     // Find empty files
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
@@ -292,10 +309,9 @@ fn empty_dirs_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.hide();
+    grid_progress.set_visible(false);
 
-    let tree_view_empty_folder_finder = gui_data.main_notebook.tree_view_empty_folder_finder.clone();
-    clean_tree_view(&tree_view_empty_folder_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
@@ -317,12 +333,11 @@ fn big_files_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.hide();
+    grid_progress.set_visible(false);
 
     let combo_box_big_files_mode = gui_data.main_notebook.combo_box_big_files_mode.clone();
     let entry_big_files_number = gui_data.main_notebook.entry_big_files_number.clone();
-    let tree_view_big_files_finder = gui_data.main_notebook.tree_view_big_files_finder.clone();
-    clean_tree_view(&tree_view_big_files_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     let big_files_mode_index = combo_box_big_files_mode.active().expect("Failed to get active search") as usize;
     let big_files_mode = BIG_FILES_CHECK_METHOD_COMBO_BOX[big_files_mode_index].check_method;
@@ -350,10 +365,9 @@ fn temporary_files_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.hide();
+    grid_progress.set_visible(false);
 
-    let tree_view_temporary_files_finder = gui_data.main_notebook.tree_view_temporary_files_finder.clone();
-    clean_tree_view(&tree_view_temporary_files_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
@@ -376,7 +390,7 @@ fn same_music_search(
     progress_data_sender: Sender<ProgressData>,
     show_dialog: &Arc<AtomicBool>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
     let check_button_music_artist: gtk4::CheckButton = gui_data.main_notebook.check_button_music_artist.clone();
     let check_button_music_title: gtk4::CheckButton = gui_data.main_notebook.check_button_music_title.clone();
@@ -384,14 +398,13 @@ fn same_music_search(
     let check_button_music_genre: gtk4::CheckButton = gui_data.main_notebook.check_button_music_genre.clone();
     let check_button_music_length: gtk4::CheckButton = gui_data.main_notebook.check_button_music_length.clone();
     let check_button_music_bitrate: gtk4::CheckButton = gui_data.main_notebook.check_button_music_bitrate.clone();
-    let tree_view_same_music_finder = gui_data.main_notebook.tree_view_same_music_finder.clone();
     let combo_box_audio_check_type = gui_data.main_notebook.combo_box_audio_check_type.clone();
     let check_button_music_approximate_comparison = gui_data.main_notebook.check_button_music_approximate_comparison.clone();
     let check_button_music_compare_only_in_title_group = gui_data.main_notebook.check_button_music_compare_only_in_title_group.clone();
     let scale_seconds_same_music = gui_data.main_notebook.scale_seconds_same_music.clone();
     let scale_similarity_same_music = gui_data.main_notebook.scale_similarity_same_music.clone();
 
-    clean_tree_view(&tree_view_same_music_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     let approximate_comparison = check_button_music_approximate_comparison.is_active();
     let comparison_only_in_title_group = check_button_music_compare_only_in_title_group.is_active();
@@ -476,15 +489,15 @@ fn broken_files_search(
     progress_data_sender: Sender<ProgressData>,
     show_dialog: &Arc<AtomicBool>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
     let check_button_broken_files_archive: gtk4::CheckButton = gui_data.main_notebook.check_button_broken_files_archive.clone();
     let check_button_broken_files_pdf: gtk4::CheckButton = gui_data.main_notebook.check_button_broken_files_pdf.clone();
     let check_button_broken_files_audio: gtk4::CheckButton = gui_data.main_notebook.check_button_broken_files_audio.clone();
     let check_button_broken_files_image: gtk4::CheckButton = gui_data.main_notebook.check_button_broken_files_image.clone();
-    let tree_view_broken_files = gui_data.main_notebook.tree_view_broken_files.clone();
+    let check_button_broken_files_video: gtk4::CheckButton = gui_data.main_notebook.check_button_broken_files_video.clone();
 
-    clean_tree_view(&tree_view_broken_files);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     let mut checked_types: CheckedTypes = CheckedTypes::NONE;
 
@@ -500,6 +513,9 @@ fn broken_files_search(
     if check_button_broken_files_archive.is_active() {
         checked_types |= CheckedTypes::ARCHIVE;
     }
+    if check_button_broken_files_video.is_active() {
+        checked_types |= CheckedTypes::VIDEO;
+    }
 
     if checked_types != CheckedTypes::NONE {
         thread::Builder::new()
@@ -549,7 +565,7 @@ fn similar_image_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
     let combo_box_image_hash_size = gui_data.main_notebook.combo_box_image_hash_size.clone();
     let combo_box_image_hash_algorithm = gui_data.main_notebook.combo_box_image_hash_algorithm.clone();
@@ -558,10 +574,9 @@ fn similar_image_search(
     let check_button_settings_similar_images_delete_outdated_cache = gui_data.settings.check_button_settings_similar_images_delete_outdated_cache.clone();
     let image_preview_similar_images = gui_data.main_notebook.image_preview_similar_images.clone();
     let scale_similarity_similar_images = gui_data.main_notebook.scale_similarity_similar_images.clone();
-    let tree_view_similar_images_finder = gui_data.main_notebook.tree_view_similar_images_finder.clone();
 
-    clean_tree_view(&tree_view_similar_images_finder);
-    image_preview_similar_images.hide();
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
+    image_preview_similar_images.set_visible(false);
 
     let hash_size_index = combo_box_image_hash_size.active().expect("Failed to get active search") as usize;
     let hash_size = IMAGES_HASH_SIZE_COMBO_BOX[hash_size_index] as u8;
@@ -581,7 +596,7 @@ fn similar_image_search(
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
         .spawn(move || {
-            let params = SimilarImagesParameters::new(similarity, hash_size, hash_alg, image_filter, ignore_same_size, loaded_commons.hide_hard_links);
+            let params = SimilarImagesParameters::new(similarity, hash_size, hash_alg, image_filter, ignore_same_size);
             let mut tool = SimilarImages::new(params);
 
             set_common_settings(&mut tool, &loaded_commons);
@@ -600,13 +615,12 @@ fn similar_video_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
     let check_button_video_ignore_same_size = gui_data.main_notebook.check_button_video_ignore_same_size.clone();
     let check_button_settings_similar_videos_delete_outdated_cache = gui_data.settings.check_button_settings_similar_videos_delete_outdated_cache.clone();
     let scale_similarity_similar_videos = gui_data.main_notebook.scale_similarity_similar_videos.clone();
-    let tree_view_similar_videos_finder = gui_data.main_notebook.tree_view_similar_videos_finder.clone();
-    clean_tree_view(&tree_view_similar_videos_finder);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     let tolerance = scale_similarity_similar_videos.value() as i32;
 
@@ -620,10 +634,13 @@ fn similar_video_search(
             let params = SimilarVideosParameters::new(
                 tolerance,
                 ignore_same_size,
-                loaded_commons.hide_hard_links,
                 DEFAULT_SKIP_FORWARD_AMOUNT,
                 DEFAULT_VID_HASH_DURATION,
                 DEFAULT_CROP_DETECT,
+                false, // Not implemented in gtk gui
+                10,    // Not implemented in gtk gui
+                false, // Not implemented in gtk gui
+                2,     // Not implemented in gtk gui
             );
             let mut tool = SimilarVideos::new(params);
 
@@ -643,10 +660,9 @@ fn bad_symlinks_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.hide();
+    grid_progress.set_visible(false);
 
-    let tree_view_invalid_symlinks = gui_data.main_notebook.tree_view_invalid_symlinks.clone();
-    clean_tree_view(&tree_view_invalid_symlinks);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
@@ -668,10 +684,9 @@ fn bad_extensions_search(
     grid_progress: &Grid,
     progress_data_sender: Sender<ProgressData>,
 ) {
-    grid_progress.show();
+    grid_progress.set_visible(true);
 
-    let tree_view_bad_extensions = gui_data.main_notebook.tree_view_bad_extensions.clone();
-    clean_tree_view(&tree_view_bad_extensions);
+    clean_tree_view(&gui_data.main_notebook.common_tree_views.get_current_subview().tree_view);
 
     thread::Builder::new()
         .stack_size(DEFAULT_THREAD_SIZE)
@@ -690,33 +705,28 @@ fn set_common_settings<T>(component: &mut T, loaded_commons: &LoadedCommonItems)
 where
     T: CommonData,
 {
-    component.set_included_directory(loaded_commons.included_directories.clone());
-    component.set_excluded_directory(loaded_commons.excluded_directories.clone());
-    component.set_reference_directory(loaded_commons.reference_directories.clone());
+    component.set_included_paths(loaded_commons.included_directories.clone());
+    component.set_excluded_paths(loaded_commons.excluded_directories.clone());
+    component.set_reference_paths(loaded_commons.reference_directories.clone());
     component.set_recursive_search(loaded_commons.recursive_search);
-    component.set_allowed_extensions(loaded_commons.allowed_extensions.clone());
-    component.set_excluded_extensions(loaded_commons.excluded_extensions.clone());
+    component.set_allowed_extensions(loaded_commons.allowed_extensions.split(',').map(str::to_string).collect());
+    component.set_excluded_extensions(loaded_commons.excluded_extensions.split(',').map(str::to_string).collect());
     component.set_excluded_items(loaded_commons.excluded_items.clone());
     component.set_exclude_other_filesystems(loaded_commons.ignore_other_filesystems);
     component.set_use_cache(loaded_commons.use_cache);
     component.set_save_also_as_json(loaded_commons.save_also_as_json);
     component.set_minimal_file_size(loaded_commons.minimal_file_size);
     component.set_maximal_file_size(loaded_commons.maximal_file_size);
+    component.set_hide_hard_links(loaded_commons.hide_hard_links);
 }
 
 #[fun_time(message = "clean_tree_view", level = "debug")]
 fn clean_tree_view(tree_view: &gtk4::TreeView) {
-    let list_store = get_list_store(tree_view);
-    let mut all_iters = Vec::new();
-    let first_iter = list_store.iter_first();
-    if let Some(first_iter) = first_iter {
-        loop {
-            all_iters.push(first_iter);
-            if !list_store.iter_next(&first_iter) {
-                break;
-            }
-        }
-    }
+    let list_store = tree_view.get_model();
+    let mut all_iters: Vec<gtk4::TreeIter> = Vec::new();
+    iter_list(&list_store, |_m, i| {
+        all_iters.push(*i);
+    });
     all_iters.reverse();
     for iter in all_iters {
         list_store.remove(&iter);
