@@ -4,7 +4,8 @@ use std::default::Default;
 use czkawka_core::common::cache::{load_cache_from_file_generalized_by_path, load_cache_from_file_generalized_by_size, save_cache_to_file_generalized};
 use czkawka_core::common::config_cache_path::get_config_cache_path;
 use czkawka_core::common::model::HashType;
-use czkawka_core::helpers::messages::Messages;
+use czkawka_core::helpers::messages::{MessageLimit, Messages};
+use czkawka_core::re_exported::HashAlg;
 use czkawka_core::tools::duplicate::DuplicateEntry;
 use czkawka_core::tools::duplicate::core::get_duplicate_cache_file;
 use czkawka_core::tools::similar_images::core::get_similar_images_cache_file;
@@ -13,12 +14,11 @@ use czkawka_core::tools::similar_videos::{DEFAULT_CROP_DETECT, DEFAULT_SKIP_FORW
 use gtk4::prelude::*;
 use gtk4::{Label, ResponseType, Window};
 use image::imageops::FilterType;
-use image_hasher::HashAlg;
 use log::error;
 
 use crate::flg;
+use crate::gtk_traits::DialogTraits;
 use crate::gui_structs::gui_data::GuiData;
-use crate::help_functions::get_dialog_box_child;
 use crate::saving_loading::{load_configuration, reset_configuration, save_configuration};
 
 pub(crate) fn connect_settings(gui_data: &GuiData) {
@@ -36,13 +36,13 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
         let button_settings = gui_data.header.button_settings.clone();
         let window_settings = gui_data.settings.window_settings.clone();
         button_settings.connect_clicked(move |_| {
-            window_settings.show();
+            window_settings.set_visible(true);
         });
 
         let window_settings = gui_data.settings.window_settings.clone();
 
         window_settings.connect_close_request(move |window| {
-            window.hide();
+            window.set_visible(false);
             glib::Propagation::Stop
         });
     }
@@ -67,7 +67,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
         let button_settings_load_configuration = gui_data.settings.button_settings_load_configuration.clone();
         let scrolled_window_errors = gui_data.scrolled_window_errors.clone();
         button_settings_load_configuration.connect_clicked(move |_| {
-            load_configuration(true, &upper_notebook, &main_notebook, &settings, &text_view_errors, &scrolled_window_errors, &Vec::new());
+            load_configuration(true, &upper_notebook, &main_notebook, &settings, &text_view_errors, &scrolled_window_errors, None);
         });
     }
     // Connect reset configuration button
@@ -88,7 +88,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
             if let Some(config_cache_path) = get_config_cache_path() {
                 if let Err(e) = open::that(&config_cache_path.cache_folder) {
                     error!("Failed to open config folder \"{}\", reason {e}", config_cache_path.cache_folder.to_string_lossy());
-                };
+                }
             } else {
                 error!("Failed to get cache folder path");
             }
@@ -101,7 +101,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
             if let Some(config_cache_path) = get_config_cache_path() {
                 if let Err(e) = open::that(&config_cache_path.config_folder) {
                     error!("Failed to open config folder \"{}\", reason {e}", config_cache_path.config_folder.to_string_lossy());
-                };
+                }
             } else {
                 error!("Failed to get settings folder path");
             }
@@ -117,7 +117,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
 
             button_settings_duplicates_clear_cache.connect_clicked(move |_| {
                 let dialog = create_clear_cache_dialog(&flg!("cache_clear_duplicates_title"), &settings_window);
-                dialog.show();
+                dialog.set_visible(true);
 
                 let text_view_errors = text_view_errors.clone();
                 let entry_settings_cache_file_minimal_size = entry_settings_cache_file_minimal_size.clone();
@@ -126,7 +126,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
                     if response_type == ResponseType::Ok {
                         let mut messages: Messages = Messages::new();
                         for use_prehash in [true, false] {
-                            for type_of_hash in &[HashType::Xxh3, HashType::Blake3, HashType::Crc32] {
+                            for type_of_hash in [HashType::Xxh3, HashType::Blake3, HashType::Crc32] {
                                 let file_name = get_duplicate_cache_file(type_of_hash, use_prehash);
                                 let (mut messages, loaded_items) = load_cache_from_file_generalized_by_size::<DuplicateEntry>(&file_name, true, &Default::default());
 
@@ -146,7 +146,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
                             }
 
                             messages.messages.push(flg!("cache_properly_cleared"));
-                            text_view_errors.buffer().set_text(messages.create_messages_text().as_str());
+                            text_view_errors.buffer().set_text(messages.create_messages_text(MessageLimit::NoLimit).as_str());
                         }
                     }
                     dialog.close();
@@ -160,22 +160,22 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
 
             button_settings_similar_images_clear_cache.connect_clicked(move |_| {
                 let dialog = create_clear_cache_dialog(&flg!("cache_clear_similar_images_title"), &settings_window);
-                dialog.show();
+                dialog.set_visible(true);
 
                 let text_view_errors = text_view_errors.clone();
 
                 dialog.connect_response(move |dialog, response_type| {
                     if response_type == ResponseType::Ok {
                         let mut messages: Messages = Messages::new();
-                        for hash_size in &[8, 16, 32, 64] {
-                            for image_filter in &[
+                        for hash_size in [8, 16, 32, 64] {
+                            for image_filter in [
                                 FilterType::Lanczos3,
                                 FilterType::CatmullRom,
                                 FilterType::Gaussian,
                                 FilterType::Nearest,
                                 FilterType::Triangle,
                             ] {
-                                for hash_alg in &[
+                                for hash_alg in [
                                     HashAlg::Blockhash,
                                     HashAlg::Gradient,
                                     HashAlg::DoubleGradient,
@@ -196,7 +196,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
                         }
 
                         messages.messages.push(flg!("cache_properly_cleared"));
-                        text_view_errors.buffer().set_text(messages.create_messages_text().as_str());
+                        text_view_errors.buffer().set_text(messages.create_messages_text(MessageLimit::NoLimit).as_str());
                     }
                     dialog.close();
                 });
@@ -209,7 +209,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
 
             button_settings_similar_videos_clear_cache.connect_clicked(move |_| {
                 let dialog = create_clear_cache_dialog(&flg!("cache_clear_similar_videos_title"), &settings_window);
-                dialog.show();
+                dialog.set_visible(true);
 
                 let text_view_errors = text_view_errors.clone();
 
@@ -225,7 +225,7 @@ pub(crate) fn connect_settings(gui_data: &GuiData) {
                         }
 
                         messages.messages.push(flg!("cache_properly_cleared"));
-                        text_view_errors.buffer().set_text(messages.create_messages_text().as_str());
+                        text_view_errors.buffer().set_text(messages.create_messages_text(MessageLimit::NoLimit).as_str());
                     }
                     dialog.close();
                 });
@@ -244,7 +244,7 @@ fn create_clear_cache_dialog(title_str: &str, window_settings: &Window) -> gtk4:
     let label3 = Label::builder().label(flg!("cache_clear_message_label_3")).build();
     let label4 = Label::builder().label(flg!("cache_clear_message_label_4")).build();
 
-    let internal_box = get_dialog_box_child(&dialog);
+    let internal_box = dialog.get_box_child();
     internal_box.append(&label);
     internal_box.append(&label2);
     internal_box.append(&label3);
