@@ -1,8 +1,10 @@
 use gtk4::prelude::*;
-use gtk4::{GestureClick, Widget};
+use gtk4::{GestureClick, Label, Widget};
 
+use crate::gtk_traits::WidgetTraits;
 use crate::gui_structs::gui_data::CZK_ICON_SORT;
-use crate::help_functions::{BottomButtonsEnum, get_custom_label_from_widget, set_icon_of_button};
+use crate::helpers::enums::BottomButtonsEnum;
+use crate::helpers::image_operations::set_icon_of_button;
 use crate::{
     CZK_ICON_COMPARE, CZK_ICON_HARDLINK, CZK_ICON_HIDE_DOWN, CZK_ICON_HIDE_UP, CZK_ICON_MOVE, CZK_ICON_SAVE, CZK_ICON_SEARCH, CZK_ICON_SELECT, CZK_ICON_SYMLINK, CZK_ICON_TRASH,
     flg,
@@ -118,14 +120,14 @@ impl GuiBottomButtons {
         }
     }
     pub(crate) fn update_language(&self) {
-        get_custom_label_from_widget(&self.buttons_search.clone()).set_text(&flg!("bottom_search_button"));
+        self.buttons_search.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_search_button"));
         self.label_buttons_select.set_text(&flg!("bottom_select_button"));
-        get_custom_label_from_widget(&self.buttons_delete.clone()).set_text(&flg!("bottom_delete_button"));
-        get_custom_label_from_widget(&self.buttons_save.clone()).set_text(&flg!("bottom_save_button"));
-        get_custom_label_from_widget(&self.buttons_symlink.clone()).set_text(&flg!("bottom_symlink_button"));
-        get_custom_label_from_widget(&self.buttons_move.clone()).set_text(&flg!("bottom_move_button"));
-        get_custom_label_from_widget(&self.buttons_hardlink.clone()).set_text(&flg!("bottom_hardlink_button"));
-        get_custom_label_from_widget(&self.buttons_compare.clone()).set_text(&flg!("bottom_compare_button"));
+        self.buttons_delete.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_delete_button"));
+        self.buttons_save.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_save_button"));
+        self.buttons_symlink.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_symlink_button"));
+        self.buttons_move.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_move_button"));
+        self.buttons_hardlink.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_hardlink_button"));
+        self.buttons_compare.get_widget_of_type::<Label>(true).set_text(&flg!("bottom_compare_button"));
         self.label_buttons_sort.set_text(&flg!("bottom_sort_button"));
 
         self.buttons_search.set_tooltip_text(Some(&flg!("bottom_search_button_tooltip")));
@@ -149,40 +151,51 @@ impl GuiBottomButtons {
 
 #[cfg(target_family = "windows")]
 fn test_hardlinks() -> bool {
-    use std::fs;
     use std::io::Write;
-    use std::path::Path;
+    use std::{env, fs};
 
-    use directories_next::ProjectDirs;
+    use rand::Rng;
 
-    let mut hardlinked = false;
+    fn try_create_hardlink(dir: &std::path::Path) -> bool {
+        use rand::RngExt;
+        let random_suffix: u32 = rand::rng().random();
+        let cache_file = dir.join(format!("czkawka_test_{}.czkawka_tmp", random_suffix));
+        let cache_file_second = dir.join(format!("czkawka_test_{}_link.czkawka_tmp", random_suffix));
 
-    // TODO - use temporary files instead of project dirs
-
-    if let Some(proj_dirs) = ProjectDirs::from("pl", "Qarmin", "Czkawka") {
-        let cache_dir = proj_dirs.cache_dir();
-        let cache_file = cache_dir.join(Path::new("GILLES_FROM_NOSE.temp"));
-        let cache_file_second = cache_dir.join(Path::new("ROBERCIG_LOWANDOWSKI.temp"));
-
-        if cache_file.exists() {
-            let _ = fs::remove_file(&cache_file);
-        }
-        if cache_file_second.exists() {
-            let _ = fs::remove_file(&cache_file_second);
-        }
-        if !cache_file.exists() && !cache_file_second.exists() {
-            if let Ok(mut file_handler) = fs::OpenOptions::new().write(true).create(true).open(&cache_file) {
-                let _ = writeln!(file_handler, "GURKA");
-            }
-            let _ = fs::hard_link(&cache_file, &cache_file_second);
-
-            if cache_file.exists() && cache_file_second.exists() {
-                hardlinked = true;
-            }
-        }
         let _ = fs::remove_file(&cache_file);
         let _ = fs::remove_file(&cache_file_second);
+
+        let result = (|| {
+            let mut file_handler = fs::File::create(&cache_file).ok()?;
+            writeln!(file_handler, "test").ok()?;
+            drop(file_handler);
+
+            czkawka_core::common::make_hard_link(&cache_file, &cache_file_second).ok()?;
+
+            if cache_file_second.exists() { Some(true) } else { None }
+        })();
+
+        let _ = fs::remove_file(&cache_file);
+        let _ = fs::remove_file(&cache_file_second);
+
+        result.unwrap_or(false)
     }
 
-    hardlinked
+    // Try home directory first
+    if let Ok(home_dir) = env::var("USERPROFILE").or_else(|_| env::var("HOME")) {
+        if let Ok(home_path) = std::path::PathBuf::from(home_dir).canonicalize() {
+            if try_create_hardlink(&home_path) {
+                return true;
+            }
+        }
+    }
+
+    // Fallback to current directory
+    if let Ok(current_dir) = env::current_dir() {
+        if try_create_hardlink(&current_dir) {
+            return true;
+        }
+    }
+
+    false
 }
