@@ -1,68 +1,28 @@
+use std::cell::RefCell;
+use std::collections::HashMap;
+use std::rc::Rc;
+
 use czkawka_core::common::model::CheckingMethod;
 use czkawka_core::localizer_core::{fnc_get_similarity_minimal, fnc_get_similarity_very_high};
 use czkawka_core::tools::big_file::SearchMode;
 use czkawka_core::tools::similar_images::SIMILAR_VALUES;
 use czkawka_core::tools::similar_images::core::get_string_from_similarity;
 use gtk4::prelude::*;
-use gtk4::{Builder, CheckButton, ComboBoxText, Entry, EventControllerKey, GestureClick, Image, Label, Notebook, Scale, ScrolledWindow, TreeView, Widget};
+use gtk4::{Builder, CheckButton, ComboBoxText, Entry, Label, Notebook, Picture, Scale, Widget};
+use log::error;
 
 use crate::flg;
+use crate::gtk_traits::WidgetTraits;
+use crate::gui_structs::common_tree_view::{CommonTreeViews, SharedModelEnum, SubView};
+use crate::gui_structs::gui_data::GuiData;
+use crate::gui_structs::gui_settings::GuiSettings;
 use crate::help_combo_box::{AUDIO_TYPE_CHECK_METHOD_COMBO_BOX, BIG_FILES_CHECK_METHOD_COMBO_BOX, DUPLICATES_CHECK_METHOD_COMBO_BOX, IMAGES_HASH_SIZE_COMBO_BOX};
-use crate::help_functions::get_all_direct_children;
-use crate::notebook_enums::{NUMBER_OF_NOTEBOOK_MAIN_TABS, NotebookMainEnum};
+use crate::notebook_enums::NotebookMainEnum;
 
 #[derive(Clone)]
 pub struct GuiMainNotebook {
     pub notebook_main: Notebook,
 
-    pub scrolled_window_duplicate_finder: ScrolledWindow,
-    pub scrolled_window_empty_folder_finder: ScrolledWindow,
-    pub scrolled_window_empty_files_finder: ScrolledWindow,
-    pub scrolled_window_temporary_files_finder: ScrolledWindow,
-    pub scrolled_window_big_files_finder: ScrolledWindow,
-    pub scrolled_window_similar_images_finder: ScrolledWindow,
-    pub scrolled_window_similar_videos_finder: ScrolledWindow,
-    pub scrolled_window_same_music_finder: ScrolledWindow,
-    pub scrolled_window_invalid_symlinks: ScrolledWindow,
-    pub scrolled_window_broken_files: ScrolledWindow,
-    pub scrolled_window_bad_extensions: ScrolledWindow,
-
-    pub tree_view_duplicate_finder: TreeView,
-    pub tree_view_empty_folder_finder: TreeView,
-    pub tree_view_empty_files_finder: TreeView,
-    pub tree_view_temporary_files_finder: TreeView,
-    pub tree_view_big_files_finder: TreeView,
-    pub tree_view_similar_images_finder: TreeView,
-    pub tree_view_similar_videos_finder: TreeView,
-    pub tree_view_same_music_finder: TreeView,
-    pub tree_view_invalid_symlinks: TreeView,
-    pub tree_view_broken_files: TreeView,
-    pub tree_view_bad_extensions: TreeView,
-
-    pub evk_tree_view_duplicate_finder: EventControllerKey,
-    pub evk_tree_view_empty_folder_finder: EventControllerKey,
-    pub evk_tree_view_empty_files_finder: EventControllerKey,
-    pub evk_tree_view_temporary_files_finder: EventControllerKey,
-    pub evk_tree_view_big_files_finder: EventControllerKey,
-    pub evk_tree_view_similar_images_finder: EventControllerKey,
-    pub evk_tree_view_similar_videos_finder: EventControllerKey,
-    pub evk_tree_view_same_music_finder: EventControllerKey,
-    pub evk_tree_view_invalid_symlinks: EventControllerKey,
-    pub evk_tree_view_broken_files: EventControllerKey,
-    pub evk_tree_view_bad_extensions: EventControllerKey,
-
-    pub gc_tree_view_duplicate_finder: GestureClick,
-    pub gc_tree_view_empty_folder_finder: GestureClick,
-    pub gc_tree_view_empty_files_finder: GestureClick,
-    pub gc_tree_view_temporary_files_finder: GestureClick,
-    pub gc_tree_view_big_files_finder: GestureClick,
-    pub gc_tree_view_similar_images_finder: GestureClick,
-    pub gc_tree_view_similar_videos_finder: GestureClick,
-    pub gc_tree_view_same_music_finder: GestureClick,
-    pub gc_tree_view_invalid_symlinks: GestureClick,
-    pub gc_tree_view_broken_files: GestureClick,
-    pub gc_tree_view_bad_extensions: GestureClick,
-
     // General
 
     // Duplicate
@@ -72,7 +32,7 @@ pub struct GuiMainNotebook {
     pub label_duplicate_hash_type: Label,
     pub check_button_duplicate_case_sensitive_name: CheckButton,
 
-    pub image_preview_duplicates: Image,
+    pub image_preview_duplicates: Picture,
 
     // Big file
     pub label_big_shown_files: Label,
@@ -97,7 +57,7 @@ pub struct GuiMainNotebook {
     pub label_image_similarity: Label,
     pub label_image_similarity_max: Label,
 
-    pub image_preview_similar_images: Image,
+    pub image_preview_similar_images: Picture,
     pub label_similar_images_minimal_similarity: Label,
 
     // Video
@@ -112,6 +72,7 @@ pub struct GuiMainNotebook {
     pub check_button_broken_files_pdf: CheckButton,
     pub check_button_broken_files_archive: CheckButton,
     pub check_button_broken_files_image: CheckButton,
+    pub check_button_broken_files_video: CheckButton,
 
     // Music
     pub check_button_music_title: CheckButton,
@@ -122,90 +83,21 @@ pub struct GuiMainNotebook {
     pub check_button_music_length: CheckButton,
     pub check_button_music_approximate_comparison: CheckButton,
     pub check_button_music_compare_only_in_title_group: CheckButton,
-    #[allow(unused)]
+    #[expect(unused)]
     pub label_audio_check_type: Label,
     pub combo_box_audio_check_type: ComboBoxText,
     pub label_same_music_seconds: Label,
     pub label_same_music_similarity: Label,
     pub scale_seconds_same_music: Scale,
     pub scale_similarity_same_music: Scale,
+
+    pub common_tree_views: CommonTreeViews,
 }
 
 impl GuiMainNotebook {
-    pub(crate) fn create_from_builder(builder: &Builder) -> Self {
+    pub(crate) fn create_from_builder(builder: &Builder, settings: &GuiSettings) -> Self {
         let notebook_main: Notebook = builder.object("notebook_main").expect("Cambalache");
 
-        let scrolled_window_duplicate_finder: ScrolledWindow = builder.object("scrolled_window_duplicate_finder").expect("Cambalache");
-        let scrolled_window_empty_folder_finder: ScrolledWindow = builder.object("scrolled_window_empty_folder_finder").expect("Cambalache");
-        let scrolled_window_empty_files_finder: ScrolledWindow = builder.object("scrolled_window_empty_files_finder").expect("Cambalache");
-        let scrolled_window_temporary_files_finder: ScrolledWindow = builder.object("scrolled_window_temporary_files_finder").expect("Cambalache");
-        let scrolled_window_big_files_finder: ScrolledWindow = builder.object("scrolled_window_big_files_finder").expect("Cambalache");
-        let scrolled_window_similar_images_finder: ScrolledWindow = builder.object("scrolled_window_similar_images_finder").expect("Cambalache");
-        let scrolled_window_similar_videos_finder: ScrolledWindow = builder.object("scrolled_window_similar_videos_finder").expect("Cambalache");
-        let scrolled_window_same_music_finder: ScrolledWindow = builder.object("scrolled_window_same_music_finder").expect("Cambalache");
-        let scrolled_window_invalid_symlinks: ScrolledWindow = builder.object("scrolled_window_invalid_symlinks").expect("Cambalache");
-        let scrolled_window_broken_files: ScrolledWindow = builder.object("scrolled_window_broken_files").expect("Cambalache");
-        let scrolled_window_bad_extensions: ScrolledWindow = builder.object("scrolled_window_bad_extensions").expect("Cambalache");
-
-        let tree_view_duplicate_finder: TreeView = TreeView::new();
-        tree_view_duplicate_finder.set_widget_name("PIERD");
-        let tree_view_empty_folder_finder: TreeView = TreeView::new();
-        let tree_view_empty_files_finder: TreeView = TreeView::new();
-        let tree_view_temporary_files_finder: TreeView = TreeView::new();
-        let tree_view_big_files_finder: TreeView = TreeView::new();
-        let tree_view_similar_images_finder: TreeView = TreeView::new();
-        let tree_view_similar_videos_finder: TreeView = TreeView::new();
-        let tree_view_same_music_finder: TreeView = TreeView::new();
-        let tree_view_invalid_symlinks: TreeView = TreeView::new();
-        let tree_view_broken_files: TreeView = TreeView::new();
-        let tree_view_bad_extensions: TreeView = TreeView::new();
-
-        let evk_tree_view_duplicate_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_duplicate_finder.add_controller(evk_tree_view_duplicate_finder.clone());
-        let evk_tree_view_empty_folder_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_empty_folder_finder.add_controller(evk_tree_view_empty_folder_finder.clone());
-        let evk_tree_view_empty_files_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_empty_files_finder.add_controller(evk_tree_view_empty_files_finder.clone());
-        let evk_tree_view_temporary_files_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_temporary_files_finder.add_controller(evk_tree_view_temporary_files_finder.clone());
-        let evk_tree_view_big_files_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_big_files_finder.add_controller(evk_tree_view_big_files_finder.clone());
-        let evk_tree_view_similar_images_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_similar_images_finder.add_controller(evk_tree_view_similar_images_finder.clone());
-        let evk_tree_view_similar_videos_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_similar_videos_finder.add_controller(evk_tree_view_similar_videos_finder.clone());
-        let evk_tree_view_same_music_finder: EventControllerKey = EventControllerKey::new();
-        tree_view_same_music_finder.add_controller(evk_tree_view_same_music_finder.clone());
-        let evk_tree_view_invalid_symlinks: EventControllerKey = EventControllerKey::new();
-        tree_view_invalid_symlinks.add_controller(evk_tree_view_invalid_symlinks.clone());
-        let evk_tree_view_broken_files: EventControllerKey = EventControllerKey::new();
-        tree_view_broken_files.add_controller(evk_tree_view_broken_files.clone());
-        let evk_tree_view_bad_extensions: EventControllerKey = EventControllerKey::new();
-        tree_view_bad_extensions.add_controller(evk_tree_view_bad_extensions.clone());
-
-        let gc_tree_view_duplicate_finder: GestureClick = GestureClick::new();
-        tree_view_duplicate_finder.add_controller(gc_tree_view_duplicate_finder.clone());
-        let gc_tree_view_empty_folder_finder: GestureClick = GestureClick::new();
-        tree_view_empty_folder_finder.add_controller(gc_tree_view_empty_folder_finder.clone());
-        let gc_tree_view_empty_files_finder: GestureClick = GestureClick::new();
-        tree_view_empty_files_finder.add_controller(gc_tree_view_empty_files_finder.clone());
-        let gc_tree_view_temporary_files_finder: GestureClick = GestureClick::new();
-        tree_view_temporary_files_finder.add_controller(gc_tree_view_temporary_files_finder.clone());
-        let gc_tree_view_big_files_finder: GestureClick = GestureClick::new();
-        tree_view_big_files_finder.add_controller(gc_tree_view_big_files_finder.clone());
-        let gc_tree_view_similar_images_finder: GestureClick = GestureClick::new();
-        tree_view_similar_images_finder.add_controller(gc_tree_view_similar_images_finder.clone());
-        let gc_tree_view_similar_videos_finder: GestureClick = GestureClick::new();
-        tree_view_similar_videos_finder.add_controller(gc_tree_view_similar_videos_finder.clone());
-        let gc_tree_view_same_music_finder: GestureClick = GestureClick::new();
-        tree_view_same_music_finder.add_controller(gc_tree_view_same_music_finder.clone());
-        let gc_tree_view_invalid_symlinks: GestureClick = GestureClick::new();
-        tree_view_invalid_symlinks.add_controller(gc_tree_view_invalid_symlinks.clone());
-        let gc_tree_view_broken_files: GestureClick = GestureClick::new();
-        tree_view_broken_files.add_controller(gc_tree_view_broken_files.clone());
-        let gc_tree_view_bad_extensions: GestureClick = GestureClick::new();
-        tree_view_bad_extensions.add_controller(gc_tree_view_bad_extensions.clone());
-
         let combo_box_duplicate_check_method: ComboBoxText = builder.object("combo_box_duplicate_check_method").expect("Cambalache");
         let combo_box_duplicate_hash_type: ComboBoxText = builder.object("combo_box_duplicate_hash_type").expect("Cambalache");
 
@@ -226,6 +118,7 @@ impl GuiMainNotebook {
         let check_button_broken_files_pdf: CheckButton = builder.object("check_button_broken_files_pdf").expect("Cambalache");
         let check_button_broken_files_archive: CheckButton = builder.object("check_button_broken_files_archive").expect("Cambalache");
         let check_button_broken_files_image: CheckButton = builder.object("check_button_broken_files_image").expect("Cambalache");
+        let check_button_broken_files_video: CheckButton = builder.object("check_button_broken_files_video").expect("Cambalache");
 
         let scale_similarity_similar_images: Scale = builder.object("scale_similarity_similar_images").expect("Cambalache");
         let scale_similarity_similar_videos: Scale = builder.object("scale_similarity_similar_videos").expect("Cambalache");
@@ -253,8 +146,8 @@ impl GuiMainNotebook {
         let label_video_similarity_max: Label = builder.object("label_video_similarity_max").expect("Cambalache");
         let label_big_files_mode: Label = builder.object("label_big_files_mode").expect("Cambalache");
 
-        let image_preview_similar_images: Image = builder.object("image_preview_similar_images").expect("Cambalache");
-        let image_preview_duplicates: Image = builder.object("image_preview_duplicates").expect("Cambalache");
+        let image_preview_similar_images: Picture = builder.object("image_preview_similar_images").expect("Cambalache");
+        let image_preview_duplicates: Picture = builder.object("image_preview_duplicates").expect("Cambalache");
 
         let label_audio_check_type: Label = builder.object("label_audio_check_type").expect("Cambalache");
         let combo_box_audio_check_type: ComboBoxText = builder.object("combo_box_audio_check_type").expect("Cambalache");
@@ -263,52 +156,31 @@ impl GuiMainNotebook {
         let scale_seconds_same_music: Scale = builder.object("scale_seconds_same_music").expect("Cambalache");
         let scale_similarity_same_music: Scale = builder.object("scale_similarity_same_music").expect("Cambalache");
 
+        #[rustfmt::skip]
+        let subviews: Vec<_> = [
+            SubView::new(builder, "scrolled_window_duplicate_finder", NotebookMainEnum::Duplicate, Some("image_preview_duplicates"), Some(settings.check_button_settings_show_preview_duplicates.clone()), SharedModelEnum::Duplicates(Rc::default())),
+            SubView::new(builder, "scrolled_window_empty_folder_finder", NotebookMainEnum::EmptyDirectories, None, None, SharedModelEnum::EmptyFolder(Rc::default())),
+            SubView::new(builder, "scrolled_window_empty_files_finder", NotebookMainEnum::EmptyFiles, None, None, SharedModelEnum::EmptyFiles(Rc::default())),
+            SubView::new(builder, "scrolled_window_temporary_files_finder", NotebookMainEnum::Temporary, None, None, SharedModelEnum::Temporary(Rc::default())),
+            SubView::new(builder, "scrolled_window_big_files_finder", NotebookMainEnum::BigFiles, None, None, SharedModelEnum::BigFile(Rc::default())),
+            SubView::new(builder, "scrolled_window_similar_images_finder", NotebookMainEnum::SimilarImages, Some("image_preview_similar_images"), Some(settings.check_button_settings_show_preview_similar_images.clone()), SharedModelEnum::SimilarImages(Rc::default())),
+            SubView::new(builder, "scrolled_window_similar_videos_finder", NotebookMainEnum::SimilarVideos, None, None, SharedModelEnum::SimilarVideos(Rc::default())),
+            SubView::new(builder, "scrolled_window_same_music_finder", NotebookMainEnum::SameMusic, None, None, SharedModelEnum::SameMusic(Rc::default())),
+            SubView::new(builder, "scrolled_window_invalid_symlinks", NotebookMainEnum::Symlinks, None, None, SharedModelEnum::Symlinks(Rc::default())),
+            SubView::new(builder, "scrolled_window_broken_files", NotebookMainEnum::BrokenFiles, None, None, SharedModelEnum::BrokenFiles(Rc::default())),
+            SubView::new(builder, "scrolled_window_bad_extensions", NotebookMainEnum::BadExtensions, None, None, SharedModelEnum::BadExtensions(Rc::default())),
+        ]
+        .into_iter()
+        .collect();
+
+        let common_tree_views = CommonTreeViews {
+            notebook_main: notebook_main.clone(),
+            subviews,
+            preview_path: Rc::new(RefCell::new(String::new())),
+        };
+
         Self {
             notebook_main,
-            scrolled_window_duplicate_finder,
-            scrolled_window_empty_folder_finder,
-            scrolled_window_empty_files_finder,
-            scrolled_window_temporary_files_finder,
-            scrolled_window_big_files_finder,
-            scrolled_window_similar_images_finder,
-            scrolled_window_similar_videos_finder,
-            scrolled_window_same_music_finder,
-            scrolled_window_invalid_symlinks,
-            scrolled_window_broken_files,
-            scrolled_window_bad_extensions,
-            tree_view_duplicate_finder,
-            tree_view_empty_folder_finder,
-            tree_view_empty_files_finder,
-            tree_view_temporary_files_finder,
-            tree_view_big_files_finder,
-            tree_view_similar_images_finder,
-            tree_view_similar_videos_finder,
-            tree_view_same_music_finder,
-            tree_view_invalid_symlinks,
-            tree_view_broken_files,
-            tree_view_bad_extensions,
-            evk_tree_view_duplicate_finder,
-            evk_tree_view_empty_folder_finder,
-            evk_tree_view_empty_files_finder,
-            evk_tree_view_temporary_files_finder,
-            evk_tree_view_big_files_finder,
-            evk_tree_view_similar_images_finder,
-            evk_tree_view_similar_videos_finder,
-            evk_tree_view_same_music_finder,
-            evk_tree_view_invalid_symlinks,
-            evk_tree_view_broken_files,
-            evk_tree_view_bad_extensions,
-            gc_tree_view_duplicate_finder,
-            gc_tree_view_empty_folder_finder,
-            gc_tree_view_empty_files_finder,
-            gc_tree_view_temporary_files_finder,
-            gc_tree_view_big_files_finder,
-            gc_tree_view_similar_images_finder,
-            gc_tree_view_similar_videos_finder,
-            gc_tree_view_same_music_finder,
-            gc_tree_view_invalid_symlinks,
-            gc_tree_view_broken_files,
-            gc_tree_view_bad_extensions,
             combo_box_duplicate_check_method,
             combo_box_duplicate_hash_type,
             label_duplicate_check_method,
@@ -340,6 +212,7 @@ impl GuiMainNotebook {
             check_button_broken_files_pdf,
             check_button_broken_files_archive,
             check_button_broken_files_image,
+            check_button_broken_files_video,
             check_button_music_title,
             check_button_music_artist,
             check_button_music_year,
@@ -354,23 +227,12 @@ impl GuiMainNotebook {
             label_same_music_similarity,
             scale_seconds_same_music,
             scale_similarity_same_music,
+            common_tree_views,
         }
     }
 
-    pub(crate) fn get_main_tree_views(&self) -> [TreeView; NUMBER_OF_NOTEBOOK_MAIN_TABS] {
-        [
-            self.tree_view_duplicate_finder.clone(),
-            self.tree_view_empty_folder_finder.clone(),
-            self.tree_view_big_files_finder.clone(),
-            self.tree_view_empty_files_finder.clone(),
-            self.tree_view_temporary_files_finder.clone(),
-            self.tree_view_similar_images_finder.clone(),
-            self.tree_view_similar_videos_finder.clone(),
-            self.tree_view_same_music_finder.clone(),
-            self.tree_view_invalid_symlinks.clone(),
-            self.tree_view_broken_files.clone(),
-            self.tree_view_bad_extensions.clone(),
-        ]
+    pub(crate) fn setup(&self, gui_data: &GuiData) {
+        self.common_tree_views.setup(gui_data);
     }
 
     pub(crate) fn update_language(&self) {
@@ -433,6 +295,9 @@ impl GuiMainNotebook {
         self.check_button_broken_files_archive.set_label(Some(&flg!("main_check_box_broken_files_archive")));
         self.check_button_broken_files_image.set_label(Some(&flg!("main_check_box_broken_files_image")));
         self.check_button_broken_files_pdf.set_label(Some(&flg!("main_check_box_broken_files_pdf")));
+        self.check_button_broken_files_video.set_label(Some(&flg!("main_check_box_broken_files_video")));
+        self.check_button_broken_files_video
+            .set_tooltip_text(Some(&flg!("main_check_box_broken_files_video_tooltip")));
 
         self.label_same_music_seconds.set_label(&flg!("same_music_seconds_label"));
         self.label_same_music_similarity.set_label(&flg!("same_music_similarity_label"));
@@ -446,26 +311,23 @@ impl GuiMainNotebook {
             let hash_size = IMAGES_HASH_SIZE_COMBO_BOX[hash_size_index];
             match hash_size {
                 8 => {
-                    self.label_similar_images_minimal_similarity.set_text(&get_string_from_similarity(&SIMILAR_VALUES[0][5], 8));
+                    self.label_similar_images_minimal_similarity.set_text(&get_string_from_similarity(SIMILAR_VALUES[0][5], 8));
                 }
                 16 => {
-                    self.label_similar_images_minimal_similarity
-                        .set_text(&get_string_from_similarity(&SIMILAR_VALUES[1][5], 16));
+                    self.label_similar_images_minimal_similarity.set_text(&get_string_from_similarity(SIMILAR_VALUES[1][5], 16));
                 }
                 32 => {
-                    self.label_similar_images_minimal_similarity
-                        .set_text(&get_string_from_similarity(&SIMILAR_VALUES[2][5], 32));
+                    self.label_similar_images_minimal_similarity.set_text(&get_string_from_similarity(SIMILAR_VALUES[2][5], 32));
                 }
                 64 => {
-                    self.label_similar_images_minimal_similarity
-                        .set_text(&get_string_from_similarity(&SIMILAR_VALUES[3][5], 64));
+                    self.label_similar_images_minimal_similarity.set_text(&get_string_from_similarity(SIMILAR_VALUES[3][5], 64));
                 }
                 _ => panic!(),
             }
         }
 
-        let vec_children: Vec<Widget> = get_all_direct_children(&self.notebook_main);
-        let vec_children: Vec<Widget> = get_all_direct_children(&vec_children[1]);
+        let vec_children: Vec<Widget> = self.notebook_main.get_all_direct_children();
+        let vec_children: Vec<Widget> = vec_children[1].get_all_direct_children();
 
         // Change name of main notebook tabs
         for (main_enum, fl_thing) in [
@@ -481,43 +343,62 @@ impl GuiMainNotebook {
             (NotebookMainEnum::BrokenFiles as usize, flg!("main_notebook_broken_files")),
             (NotebookMainEnum::BadExtensions as usize, flg!("main_notebook_bad_extensions")),
         ] {
-            self.notebook_main
-                .tab_label(&vec_children[main_enum])
-                .expect("Tab label must be set")
-                .downcast::<Label>()
-                .expect("Tab label must be a label")
-                .set_text(&fl_thing);
+            let tabel = self.notebook_main.tab_label(&vec_children[main_enum]);
+
+            if let Some(tabel) = tabel {
+                tabel.downcast::<Label>().expect("Tab label must be a label").set_text(&fl_thing);
+            } else {
+                error!("Tab label for main notebook not found for enum {main_enum:?}, message {fl_thing:?}");
+            }
         }
 
         // Change names of columns
-        let names_of_columns = [
+        let mut names_of_columns: HashMap<NotebookMainEnum, Vec<String>> = HashMap::new();
+
+        names_of_columns.insert(
+            NotebookMainEnum::Duplicate,
             vec![
                 flg!("main_tree_view_column_size"),
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Duplicates
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::EmptyDirectories,
             vec![
                 flg!("main_tree_view_column_folder_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Empty Folders
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::BigFiles,
             vec![
                 flg!("main_tree_view_column_size"),
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Big files
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::EmptyFiles,
             vec![
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Empty files
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::Temporary,
             vec![
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Temporary Files
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::SimilarImages,
             vec![
                 flg!("main_tree_view_column_similarity"),
                 flg!("main_tree_view_column_size"),
@@ -525,13 +406,23 @@ impl GuiMainNotebook {
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Similar Images
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::SimilarVideos,
             vec![
                 flg!("main_tree_view_column_size"),
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Similar Videos
+                flg!("main_tree_view_column_fps"),
+                flg!("main_tree_view_column_codec"),
+                flg!("main_tree_view_column_bitrate"),
+                flg!("main_tree_view_column_dimensions"),
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::SameMusic,
             vec![
                 flg!("main_tree_view_column_size"),
                 flg!("main_tree_view_column_file_name"),
@@ -543,35 +434,51 @@ impl GuiMainNotebook {
                 flg!("main_tree_view_column_genre"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_modification"),
-            ], // Music Duplicates
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::Symlinks,
             vec![
                 flg!("main_tree_view_column_symlink_file_name"),
                 flg!("main_tree_view_column_symlink_folder"),
                 flg!("main_tree_view_column_destination_path"),
                 flg!("main_tree_view_column_type_of_error"),
                 flg!("main_tree_view_column_modification"),
-            ], // Invalid Symlinks
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::BrokenFiles,
             vec![
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_type_of_error"),
                 flg!("main_tree_view_column_modification"),
-            ], // Broken Files
+            ],
+        );
+        names_of_columns.insert(
+            NotebookMainEnum::BadExtensions,
             vec![
                 flg!("main_tree_view_column_file_name"),
                 flg!("main_tree_view_column_path"),
                 flg!("main_tree_view_column_current_extension"),
                 flg!("main_tree_view_column_proper_extensions"),
-                flg!("main_tree_view_column_modification"),
-            ], // Broken Files
-        ];
+                // flg!("main_tree_view_column_modification"), // TODO - too much data?
+            ],
+        );
 
-        for (notebook_index, tree_view) in self.get_main_tree_views().iter().enumerate() {
-            for (column_index, column) in tree_view.columns().iter().enumerate() {
-                if column_index == 0 {
-                    continue; // Selection button
-                }
-                column.set_title(&names_of_columns[notebook_index][column_index - 1]);
+        for (key_enum, columns_names) in names_of_columns {
+            let s = &self.common_tree_views.get_subview(key_enum);
+
+            // Skipping first column because it is selection button
+            assert_eq!(
+                columns_names.len() + 1,
+                s.tree_view.columns().len(),
+                "Number of columns in tree view and names do not match for {:?}, tree_view - {:?}",
+                key_enum,
+                s.tree_view.widget_name()
+            );
+            for (column, name) in s.tree_view.columns().iter().skip(1).zip(columns_names.iter()) {
+                column.set_title(name);
             }
         }
 
