@@ -4,11 +4,11 @@ use std::mem;
 use std::sync::{LazyLock, RwLock, RwLockWriteGuard};
 
 use czkawka_core::TOOLS_NUMBER;
-use log::{debug, error, trace};
+use log::{error, trace};
 use slint::{ComponentHandle, Model, ModelRc, VecModel};
 
 use crate::common::{connect_i32_into_u64, split_u64_into_i32s};
-use crate::{ActiveTab, Callabler, GuiState, MainListModel, MainWindow};
+use crate::{ActiveTab, Callabler, GuiState, MainWindow, SingleMainListModel};
 
 const SELECTED_ROWS_LIMIT: usize = 1000;
 
@@ -24,23 +24,37 @@ pub(crate) struct SelectionData {
 
 pub(crate) static TOOLS_SELECTION: LazyLock<RwLock<HashMap<ActiveTab, SelectionData>>> = LazyLock::new(|| RwLock::new(HashMap::new()));
 
-pub(crate) fn reset_selection(app: &MainWindow, reset_all_selection: bool) {
+// pub(crate) fn get_selection_data(active_tab: ActiveTab) -> SelectionData {
+//     let lock = TOOLS_SELECTION.read().expect("Selection data is not initialized or is poisoned");
+//     let keys = lock.keys().cloned().collect::<Vec<_>>();
+//     lock.get(&active_tab)
+//         .unwrap_or_else(|| panic!("Failed to get selection data for tab {active_tab:?} - {keys:?}"))
+//         .clone()
+// }
+
+pub(crate) fn reset_selection(app: &MainWindow, active_tab: ActiveTab, reset_all_selection: bool) {
     if reset_all_selection {
-        let active_tab = app.global::<GuiState>().get_active_tab();
         let mut lock = get_write_selection_lock();
-        let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
+        let keys = lock.keys().cloned().collect::<Vec<_>>();
+        let selection = lock
+            .get_mut(&active_tab)
+            .unwrap_or_else(|| panic!("Failed to get selection data for tab {active_tab:?} - {keys:?}"));
         selection.selected_rows.clear();
+        selection.number_of_selected_rows = 0;
         selection.exceeded_limit = false;
     }
 
-    app.invoke_reset_selection();
+    app.invoke_reset_selection(active_tab);
 }
 
 // E.g. when sorting things, selected rows in vector, may be invalid
 // So we need to recalculate them
-pub(crate) fn recalculate_small_selection_if_needed(model: &ModelRc<MainListModel>, active_tab: ActiveTab) {
+pub(crate) fn recalculate_small_selection_if_needed(model: &ModelRc<SingleMainListModel>, active_tab: ActiveTab) {
     let mut lock = get_write_selection_lock();
-    let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
+    let keys = lock.keys().cloned().collect::<Vec<_>>();
+    let selection = lock
+        .get_mut(&active_tab)
+        .unwrap_or_else(|| panic!("Failed to get selection data for tab {active_tab:?} - {keys:?}"));
 
     if selection.exceeded_limit || selection.selected_rows.is_empty() {
         return;
@@ -73,6 +87,9 @@ pub(crate) fn initialize_selection_struct() {
         ActiveTab::InvalidSymlinks,
         ActiveTab::BrokenFiles,
         ActiveTab::BadExtensions,
+        ActiveTab::BadNames,
+        ActiveTab::ExifRemover,
+        ActiveTab::VideoOptimizer,
     ];
 
     let map: HashMap<_, _> = tools.into_iter().map(|tool| (tool, SelectionData::default())).collect();
@@ -90,7 +107,7 @@ pub(crate) fn initialize_selection_struct() {
 //     selection.read().expect("Failed to lock selection data")
 // }
 fn get_write_selection_lock() -> RwLockWriteGuard<'static, HashMap<ActiveTab, SelectionData>> {
-    TOOLS_SELECTION.write().expect("Selection data is not initialized")
+    TOOLS_SELECTION.write().expect("Selection data is not initialized or is poisoned")
 }
 
 impl Hash for ActiveTab {
@@ -117,15 +134,14 @@ pub(crate) fn connect_row_selections(app: &MainWindow) {
     selection::reverse_checked_on_selection(app); // Space
     selection::reverse_selection_on_specific_item(app); // CTRL + LMB
     selection::select_items_with_shift(app); // SHIFT + LMB
-    opener::open_selected_item(app);
-    opener::open_parent_of_selected_item(app);
     opener::open_provided_item(app);
+    opener::open_provided_parent_item(app);
     opener::connect_on_open_item(app);
     checker::change_number_of_checked_items(app);
 }
 
 mod opener {
-    use super::*;
+    use super::{Callabler, ComponentHandle, GuiState, MainWindow, Model, error};
 
     pub(crate) fn connect_on_open_item(app: &MainWindow) {
         app.global::<Callabler>().on_open_item(move |path| {
@@ -133,7 +149,7 @@ mod opener {
         });
         app.global::<Callabler>().on_open_parent(move |path| {
             let Some(parent_path) = std::path::Path::new(&path).parent() else {
-                return error!("Failed to get parent path for {path}");
+                return error!("Failed to get parent path for \"{path}\"");
             };
             open_item_simple(&parent_path.to_string_lossy());
         });
@@ -142,7 +158,7 @@ mod opener {
     fn open_item_simple(path_to_open: &str) {
         if let Err(e) = open::that(path_to_open) {
             error!("Failed to open file: {e}");
-        };
+        }
     }
 
     fn open_item(app: &MainWindow, items_path_str: &[usize], id: usize) {
@@ -152,53 +168,29 @@ mod opener {
             .row_data(id)
             .unwrap_or_else(|| panic!("Failed to get row data with id {id}, with model {} items", model.row_count()));
 
+        let get_debug_crash_data = || {
+            format!(
+                "Model data str - {} - cannot find path/name at index/es - {:?}, active tab - {active_tab:?}",
+                model_data.val_str.iter().map(|e| e.to_string()).collect::<Vec<_>>().join(", "),
+                items_path_str
+            )
+        };
+
         let path_to_open = if items_path_str.len() == 1 {
-            format!("{}", model_data.val_str.iter().nth(items_path_str[0]).expect("Cannot find path"))
+            format!(
+                "{}",
+                model_data.val_str.iter().nth(items_path_str[0]).unwrap_or_else(|| panic!("{}", get_debug_crash_data()))
+            )
         } else {
             format!(
                 "{}/{}",
-                model_data.val_str.iter().nth(items_path_str[0]).expect("Cannot find path"),
-                model_data.val_str.iter().nth(items_path_str[1]).expect("Cannot find name")
+                model_data.val_str.iter().nth(items_path_str[0]).unwrap_or_else(|| panic!("{}", get_debug_crash_data())),
+                model_data.val_str.iter().nth(items_path_str[1]).unwrap_or_else(|| panic!("{}", get_debug_crash_data()))
             )
         };
         open_item_simple(&path_to_open);
     }
 
-    fn open_selected_items(app: &MainWindow, items_path_str: &[usize]) {
-        let active_tab = app.global::<GuiState>().get_active_tab();
-        let mut lock = get_write_selection_lock();
-        let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
-
-        if selection.selected_rows.len() == 1 {
-            let id = selection.selected_rows[0];
-            open_item(app, items_path_str, id);
-        } else {
-            if selection.selected_rows.is_empty() {
-                debug!("Failed to open selected item, because there is no selected item");
-            } else {
-                debug!("Failed to open selected item, because there is more than one selected item");
-            }
-        }
-    }
-
-    pub(crate) fn open_selected_item(app: &MainWindow) {
-        let a = app.as_weak();
-        app.global::<Callabler>().on_row_open_selected_item(move || {
-            let app = a.upgrade().expect("Failed to upgrade app :(");
-            let active_tab = app.global::<GuiState>().get_active_tab();
-            open_selected_items(&app, &[active_tab.get_str_path_idx(), active_tab.get_str_name_idx()]);
-        });
-    }
-
-    pub(crate) fn open_parent_of_selected_item(app: &MainWindow) {
-        let a = app.as_weak();
-        app.global::<Callabler>().on_row_open_parent_of_selected_item(move || {
-            let app = a.upgrade().expect("Failed to upgrade app :(");
-            let active_tab = app.global::<GuiState>().get_active_tab();
-            open_selected_items(&app, &[active_tab.get_str_path_idx()]);
-        });
-    }
-
     pub(crate) fn open_provided_item(app: &MainWindow) {
         let a = app.as_weak();
         app.global::<Callabler>().on_row_open_item_with_index(move |idx| {
@@ -208,11 +200,43 @@ mod opener {
             open_item(&app, &[active_tab.get_str_path_idx(), active_tab.get_str_name_idx()], idx as usize);
         });
     }
+
+    pub(crate) fn open_provided_parent_item(app: &MainWindow) {
+        let a = app.as_weak();
+        app.global::<Callabler>().on_row_open_parent_item_with_index(move |idx| {
+            let app = a.upgrade().expect("Failed to upgrade app :(");
+            let active_tab = app.global::<GuiState>().get_active_tab();
+
+            open_item(&app, &[active_tab.get_str_path_idx()], idx as usize);
+        });
+    }
 }
 mod selection {
-    use super::*;
+    use slint::ModelRc;
+
+    use super::{
+        Callabler, ComponentHandle, GuiState, MainWindow, Model, get_write_selection_lock, reverse_selection_of_item_with_id, row_select_items_with_shift,
+        rows_deselect_all_by_mode, rows_reverse_checked_selection, rows_select_all_by_mode, trace,
+    };
+    use crate::SingleMainListModel;
+    use crate::connect_row_selection::SelectionData;
     use crate::connect_row_selection::checker::change_number_of_enabled_items;
 
+    fn validate_selection_and_model(selection: &SelectionData, model: &ModelRc<SingleMainListModel>) {
+        assert!(
+            selection.number_of_selected_rows == selection.selected_rows.len() || selection.exceeded_limit,
+            "Number of selected rows {} should be equal to length of selected rows vector {} if not exceeded limit",
+            selection.number_of_selected_rows,
+            selection.selected_rows.len()
+        );
+        assert!(
+            model.row_count() >= selection.number_of_selected_rows,
+            "Number of model items {} should be bigger than number of selected items {}",
+            model.row_count(),
+            selection.number_of_selected_rows
+        );
+    }
+
     pub(crate) fn connect_select_all_rows(app: &MainWindow) {
         let a = app.as_weak();
         app.global::<Callabler>().on_row_select_all(move || {
@@ -224,9 +248,11 @@ mod selection {
             let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
             let model = active_tab.get_tool_model(&app);
 
+            validate_selection_and_model(selection, &model);
             if let Some(new_model) = rows_select_all_by_mode(selection, &model) {
+                validate_selection_and_model(selection, &new_model);
                 active_tab.set_tool_model(&app, new_model);
-            };
+            }
         });
     }
 
@@ -241,6 +267,7 @@ mod selection {
 
             {
                 let model = active_tab.get_tool_model(&app);
+                validate_selection_and_model(selection, &model);
 
                 if let Some(new_model) = rows_deselect_all_by_mode(selection, &model) {
                     active_tab.set_tool_model(&app, new_model);
@@ -250,6 +277,7 @@ mod selection {
             // needs to get model again, because it could be replaced
             let model = active_tab.get_tool_model(&app);
             reverse_selection_of_item_with_id(selection, &model, id as usize);
+            validate_selection_and_model(selection, &model);
         });
     }
 
@@ -263,11 +291,15 @@ mod selection {
             let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
             let model = active_tab.get_tool_model(&app);
 
+            validate_selection_and_model(selection, &model);
+
             let (checked_items, unchecked_items, new_model) = rows_reverse_checked_selection(selection, &model);
             if let Some(new_model) = new_model {
                 active_tab.set_tool_model(&app, new_model);
             }
+
             change_number_of_enabled_items(&app, active_tab, checked_items as i64 - unchecked_items as i64);
+            validate_selection_and_model(selection, &active_tab.get_tool_model(&app));
         });
     }
     pub(crate) fn reverse_selection_on_specific_item(app: &MainWindow) {
@@ -279,8 +311,9 @@ mod selection {
             let mut lock = get_write_selection_lock();
             let selection = lock.get_mut(&active_tab).expect("Failed to get selection data");
             let model = active_tab.get_tool_model(&app);
-
+            validate_selection_and_model(selection, &model);
             reverse_selection_of_item_with_id(selection, &model, id as usize);
+            validate_selection_and_model(selection, &model);
         });
     }
 
@@ -299,15 +332,17 @@ mod selection {
             assert!((first_idx as usize) < model.row_count());
             assert!((second_idx as usize) < model.row_count());
 
+            validate_selection_and_model(selection, &model);
             if let Some(new_model) = row_select_items_with_shift(selection, &model, (first_idx as usize, second_idx as usize)) {
+                validate_selection_and_model(selection, &new_model);
                 active_tab.set_tool_model(&app, new_model);
-            };
+            }
         });
     }
 }
 
 pub(crate) mod checker {
-    use super::*;
+    use super::{ActiveTab, Callabler, ComponentHandle, GuiState, MainWindow, connect_i32_into_u64, split_u64_into_i32s, trace};
 
     pub(crate) fn change_number_of_checked_items(app: &MainWindow) {
         let a = app.as_weak();
@@ -367,6 +402,18 @@ pub(crate) mod checker {
                 app.global::<GuiState>().set_selected_results_bad_extensions(it1);
                 app.global::<GuiState>().set_selected_results_bad_extensions2(it2);
             }
+            ActiveTab::BadNames => {
+                app.global::<GuiState>().set_selected_results_bad_names(it1);
+                app.global::<GuiState>().set_selected_results_bad_names2(it2);
+            }
+            ActiveTab::ExifRemover => {
+                app.global::<GuiState>().set_selected_results_exif_remover(it1);
+                app.global::<GuiState>().set_selected_results_exif_remover2(it2);
+            }
+            ActiveTab::VideoOptimizer => {
+                app.global::<GuiState>().set_selected_results_video_optimizer(it1);
+                app.global::<GuiState>().set_selected_results_video_optimizer2(it2);
+            }
             _ => unreachable!("Current tab is not a tool that has enabled items"),
         }
     }
@@ -425,6 +472,18 @@ pub(crate) mod checker {
                 app.global::<GuiState>().get_selected_results_bad_extensions(),
                 app.global::<GuiState>().get_selected_results_bad_extensions2(),
             ),
+            ActiveTab::BadNames => (
+                app.global::<GuiState>().get_selected_results_bad_names(),
+                app.global::<GuiState>().get_selected_results_bad_names2(),
+            ),
+            ActiveTab::ExifRemover => (
+                app.global::<GuiState>().get_selected_results_exif_remover(),
+                app.global::<GuiState>().get_selected_results_exif_remover2(),
+            ),
+            ActiveTab::VideoOptimizer => (
+                app.global::<GuiState>().get_selected_results_video_optimizer(),
+                app.global::<GuiState>().get_selected_results_video_optimizer2(),
+            ),
             _ => unreachable!("Current tab is not a tool that has enabled items"),
         };
         connect_i32_into_u64(it1, it2)
@@ -444,7 +503,7 @@ pub(crate) mod checker {
 // Deselect
 //
 
-fn rows_deselect_all_by_mode(selection: &mut SelectionData, model: &ModelRc<MainListModel>) -> Option<ModelRc<MainListModel>> {
+fn rows_deselect_all_by_mode(selection: &mut SelectionData, model: &ModelRc<SingleMainListModel>) -> Option<ModelRc<SingleMainListModel>> {
     let new_model = if selection.exceeded_limit {
         Some(rows_deselect_all_selected_by_replacing_models(model))
     } else if !selection.selected_rows.is_empty() {
@@ -462,7 +521,7 @@ fn rows_deselect_all_by_mode(selection: &mut SelectionData, model: &ModelRc<Main
     new_model
 }
 
-fn rows_deselect_all_selected_one_by_one(model: &ModelRc<MainListModel>, selection: &SelectionData) {
+fn rows_deselect_all_selected_one_by_one(model: &ModelRc<SingleMainListModel>, selection: &SelectionData) {
     for id in &selection.selected_rows {
         let mut model_data = model
             .row_data(*id)
@@ -473,7 +532,7 @@ fn rows_deselect_all_selected_one_by_one(model: &ModelRc<MainListModel>, selecti
     }
 }
 
-fn rows_deselect_all_selected_by_replacing_models(model: &ModelRc<MainListModel>) -> ModelRc<MainListModel> {
+fn rows_deselect_all_selected_by_replacing_models(model: &ModelRc<SingleMainListModel>) -> ModelRc<SingleMainListModel> {
     let new_model = model
         .iter()
         .map(|mut row| {
@@ -487,7 +546,13 @@ fn rows_deselect_all_selected_by_replacing_models(model: &ModelRc<MainListModel>
 //
 // Select All
 //
-fn rows_select_all_by_mode(selection: &mut SelectionData, model: &ModelRc<MainListModel>) -> Option<ModelRc<MainListModel>> {
+fn rows_select_all_by_mode(selection: &mut SelectionData, model: &ModelRc<SingleMainListModel>) -> Option<ModelRc<SingleMainListModel>> {
+    assert!(
+        model.row_count() >= selection.number_of_selected_rows,
+        "Number of model items {} should be bigger than number of selected items {}",
+        model.row_count(),
+        selection.number_of_selected_rows
+    );
     let new_model = if model.row_count() - selection.number_of_selected_rows > 100 {
         rows_select_all_by_replacing_models(selection, model)
     } else {
@@ -513,7 +578,7 @@ fn rows_select_all_by_mode(selection: &mut SelectionData, model: &ModelRc<MainLi
     new_model
 }
 
-fn rows_select_all_one_by_one(model: &ModelRc<MainListModel>) {
+fn rows_select_all_one_by_one(model: &ModelRc<SingleMainListModel>) {
     let items_to_update = model.iter().filter(|e| !e.selected_row && !e.header_row).count();
     trace!("[FAST][ONE_BY_ONE] select all {}/{} items", items_to_update, model.row_count());
     for id in 0..model.row_count() {
@@ -534,7 +599,7 @@ fn rows_select_all_one_by_one(model: &ModelRc<MainListModel>) {
     }
 }
 
-fn rows_select_all_by_replacing_models(selection: &SelectionData, model: &ModelRc<MainListModel>) -> Option<ModelRc<MainListModel>> {
+fn rows_select_all_by_replacing_models(selection: &SelectionData, model: &ModelRc<SingleMainListModel>) -> Option<ModelRc<SingleMainListModel>> {
     // May happen with simple models, but for more advanced with header rows, we need something like "selection.all_items_selected"
     if selection.number_of_selected_rows == model.row_count() {
         trace!(
@@ -558,7 +623,7 @@ fn rows_select_all_by_replacing_models(selection: &SelectionData, model: &ModelR
 //
 // Reverse selection and selecting
 //
-fn reverse_selection_of_item_with_id(selection: &mut SelectionData, model: &ModelRc<MainListModel>, id: usize) {
+fn reverse_selection_of_item_with_id(selection: &mut SelectionData, model: &ModelRc<SingleMainListModel>, id: usize) {
     let mut model_data = model
         .row_data(id)
         .unwrap_or_else(|| panic!("Failed to get row data with id {id}, with model {} items", model.row_count()));
@@ -587,7 +652,7 @@ fn reverse_selection_of_item_with_id(selection: &mut SelectionData, model: &Mode
     }
 }
 
-fn row_select_items_with_shift(selection: &mut SelectionData, model: &ModelRc<MainListModel>, indexes: (usize, usize)) -> Option<ModelRc<MainListModel>> {
+fn row_select_items_with_shift(selection: &mut SelectionData, model: &ModelRc<SingleMainListModel>, indexes: (usize, usize)) -> Option<ModelRc<SingleMainListModel>> {
     let (smaller_idx, bigger_idx) = if indexes.0 < indexes.1 { (indexes.0, indexes.1) } else { (indexes.1, indexes.0) };
 
     if bigger_idx - smaller_idx > SELECTED_ROWS_LIMIT || selection.exceeded_limit {
@@ -662,7 +727,7 @@ fn row_select_items_with_shift(selection: &mut SelectionData, model: &ModelRc<Ma
     }
 }
 
-fn rows_reverse_checked_selection(selection: &SelectionData, model: &ModelRc<MainListModel>) -> (u64, u64, Option<ModelRc<MainListModel>>) {
+fn rows_reverse_checked_selection(selection: &SelectionData, model: &ModelRc<SingleMainListModel>) -> (u64, u64, Option<ModelRc<SingleMainListModel>>) {
     let (mut checked_items, mut unchecked_items) = (0, 0);
 
     if selection.exceeded_limit {
@@ -706,7 +771,8 @@ fn rows_reverse_checked_selection(selection: &SelectionData, model: &ModelRc<Mai
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_common::{create_model_from_model_vec, get_model_vec};
+    use crate::common::create_model_from_model_vec;
+    use crate::test_common::get_model_vec;
 
     #[test]
     fn rows_deselect_all_by_mode_with_exceeded_limit() {
@@ -764,7 +830,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: false,
         };
 
@@ -812,7 +878,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: false,
         };
 
@@ -837,7 +903,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: true,
         };
 
@@ -865,7 +931,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: false,
         };
 
@@ -903,7 +969,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: false,
         };
 
@@ -928,7 +994,7 @@ mod tests {
 
         let mut selection = SelectionData {
             number_of_selected_rows: 0,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: false,
         };
 
@@ -978,7 +1044,7 @@ mod tests {
 
         let selection = SelectionData {
             number_of_selected_rows: 2,
-            selected_rows: vec![],
+            selected_rows: Vec::new(),
             exceeded_limit: true,
         };
 
